# MTR's config file for tkabber

set ssj::options(sign-traffic)    1
set ssj::options(encrypt-traffic) 1

set debug_lvls                    [list zerobot]
set debug_winP                    1

set raise_new_tab                 0

set show_splash_window            1

set use_ispell                    1

proc debugmsg {module msg} {
    global debug_lvls w
    global debug_fd debug_winP

    if {![info exists debug_fd]} {
        catch { file rename -force -- ~/.tkabber/tkabber.log \
                                      ~/.tkabber/tkabber0.log }
        set debug_fd [open ~/.tkabber/tkabber.log \
                           { WRONLY CREAT TRUNC APPEND }]
        fconfigure $debug_fd -buffering line
    }

    puts $debug_fd [format "%s %-12.12s %s" \
                           [clock format [clock seconds] -format "%m/%d %T"] \
                           $module $msg]

    if {([lsearch -exact $debug_lvls $module] < 0) || (!$debug_winP)} {
        return
    }

    set dw $w.debug

    if {![winfo exists $dw]} {
        add_win $dw -title Debug -tabtitle debug -class Chat

        [ScrolledWindow $dw.sw] setwidget \
            [text $dw.body -yscrollcommand [list $dw.scroll set]]

        pack $dw.sw -side bottom -fill both -expand yes

        $dw.body tag configure module \
            -foreground [option get $dw theyforeground Chat]
        $dw.body tag configure proc   \
            -foreground [option get $dw meforeground Chat]
        $dw.body tag configure error  \
            -foreground [option get $dw errforeground Chat]
    }

    $dw.body insert end [format "%-12.12s" $module] module " "
    set tag normal
    switch -- $module {
        jlib {
            if {[set x [string first "(jlib::" $msg]] > 0} {
                set tag error
            }
            if {[set y [string first ")" $msg]] > 0} {
                $dw.body insert end \
                         [string range $msg [expr $x+7] [expr $y-1]] proc \
                         "\n"
                set msg [string trimleft \
                                [string range $msg [expr $y+1] end]]
            }
        }

        default {
        }
    }
    $dw.body insert end [string trimright $msg] $tag
    $dw.body insert end "\n"

    if {!$chat::options(stop_scroll)} {
        $dw.body see end
    }
}

set load_default_xrdb 0
option readfile ~/src/tkabber/tkabber/examples/dark.xrdb userDefault
option add *font fixed userDefault
option add Tkabber.geometry =788x550-0+0 userDefault


proc postload {} {
# the autoaway module

    set plugins::autoaway::options(drop_priority) 0


# the conferenceinfo module

    set plugins::conferenceinfo::options(autoask)         1
    set plugins::conferenceinfo::options(interval)      300
    set plugins::conferenceinfo::options(err_interval) 3600


# the emoticon module

    emoteicons::load_dir ~/.tkabber/emoticons/rythmbox


# the georoster module

    source ~/src/tkabber/tkabber-plugins/georoster/georoster.tcl
    source ~/.tkabber/proprietary/georoster-prop.tcl

    set georoster::options(automatic)  1
    set georoster::options(check_vcard) 1
    set georoster::options(showcities) none

if {0} {
    namespace eval georoster {
	variable options

        set options(mapfile)    ~/src/tkabber/tkabber-plugins/georoster/bwmap4.gif

	proc lo {x y} {expr {($x*2 - 649)*18/65 + 10}}
	proc la {x y} {expr {(371 - $y*2)*9/40}}

	proc x {lo la} {expr {(649+(($lo-10)*65/18))/2}}
	proc y {lo la} {expr {(371-($la * 40/9))/2}}
    }
} else {
    set georoster::options(mapfile)    \
        ~/src/tkabber/tkabber-plugins/georoster/darkmap.gif
}


# the ispell module

    set plugins::ispell::options(executable) ispell


# the login module

    global loginconf loginconf1 loginconf2

    set loginconf1(profile)     Default
    set loginconf1(user)        mrose

    set loginconf(password)      ""
    set loginconf(resource)      tkabber
    set loginconf(server)        jabber.dbc.mtview.ca.us
    set loginconf(port)          5222
    set loginconf(priority)      8
    set loginconf(usessl)        1
    set loginconf(sslport)       5223
    set loginconf(useproxy)      0
    set loginconf(httpproxy)     localhost
    set loginconf(httpproxyport) 3128

    array set loginconf          [array get loginconf1]


# the message module

    set message::options(headlines,cache) 1
    set message::options(headlines,multiple) 1


# the roster module

    set roster::show_only_online              1
    set roster::show_transport_icons          1
    set roster::show_transport_user_icons     1
    set roster::roster(collapsed,Conferences) 1
    set roster::roster(collapsed,IETF)        1
    set roster::roster(collapsed,bots)        1
    set roster::roster(collapsed,cvideo)      1
    set roster::roster(collapsed,transports)  1
    set roster::roster(collapsed,Undefined)   1

# special key bindings (due to KDE)

    bind . <Alt-Left>      {tab_move .nb -1}
    bind . <Alt-Right>     {tab_move .nb  1}
    bind . <Control-Left>  {current_tab_move .nb -1}
    bind . <Control-Right> {current_tab_move .nb  1}
}


proc menuload {menudesc} {
    global version

return                                                                        \
[list                                                                         \
&TKabber "" "" 1                                                              \
      [list                                                                   \
            [list cascad  Presence                            "" presence 1   \
                  [list                                                       \
                        [list command Online               "" "" ""           \
                              -command "set userstatus available"]            \
                        [list command "Free to chat"          "" "" ""        \
                                      -command "set userstatus chat"]         \
                        [list command Away                    "" "" ""        \
                                      -command "set userstatus away"]         \
                        [list command "Extended Away"         "" "" ""        \
                                      -command "set userstatus xa"]           \
                        [list command Busy                    "" "" ""        \
                                      -command "set userstatus dnd"]          \
                        [list command Invisible               "" "" ""        \
                                      -command "set userstatus invisible"]    \
                  ]                                                           \
            ]                                                                 \
            [list command "&Log in..."                        "" "" "Ctrl L"  \
                              -command show_login_dialog]                     \
            [list command "Log &out"                          "" "" ""        \
                              -command logout]                                \
            [list command "Log out &with Reason..."           "" "" ""        \
                              -command show_logout_dialog]                    \
            separator                                                         \
            [list command "Message archive..."                "" "" ""        \
                              -command message_archive::show_archive]         \
            separator                                                         \
            [list command "&My Information..."                "" "" ""        \
                              -command {                                      
userinfo::open ${loginconf(user)}@$loginconf(server)/$loginconf(resource) 1
}]                                                                            \
            [list command "Change password..."                "" "" ""        \
                            -command change_password_dialog]                  \
            [list cascad  &Preferences                        "" preferences 1\
                [list                                                         \
                      [list checkbutton "Announce avatar"                     \
                                                              "" "" ""        \
                            -variable avatar::options(announce)]              \
                      [list checkbutton "Allow avatar to be downloaded"       \
                                                              "" "" ""        \
                            -variable avatar::options(share)]                 \
                      [list command "Store avatar on server"  "" "" ""        \
                            -command avatar::store_on_server]                 \
                      separator                                               \
                      [list checkbutton "Smart autoscroll"    "" "" ""        \
                            -variable chat::options(smart_scroll)]            \
                      [list checkbutton "Stop autoscroll"     "" "" ""        \
                            -variable chat::options(stop_scroll)]             \
                      [list checkbutton "Default to chat"     "" "" ""        \
                            -onvalue  chat                                    \
                            -offvalue message                                 \
                            -variable chat::options(default_message_type)]    \
                      separator                                               \
                      [list checkbutton "Generate event messages" "" "" ""    \
                            -variable muc::options(gen_events)]               \
                      separator                                               \
                      [list checkbutton "Encrypt traffic (when possible)"     \
                                                              "" "" ""        \
                            -variable ssj::options(encrypt-traffic)]          \
                      [list checkbutton "Sign traffic"        "" "" ""        \
                            -variable ssj::options(sign-traffic)]             \
                      separator                                               \
                      [list cascad  JIDlink                   "" jidlink 1 ""]\
                      separator                                               \
                      [list cascad  &Roster                   "" roster  1    \
                            [list                                             \
                                  [list checkbutton                           \
                                     "Hide &Offline contacts" "" "" "Ctrl H"  \
                                     -command roster::redraw                  \
                                     -variable roster::show_only_online]      \
                                  [list checkbutton                           \
                                     "Use aliases"            "" "" ""        \
                                     -command roster::redraw                  \
                                     -variable roster::use_aliases]           \
                            ]                                                 \
                      ]                                                       \
                      separator                                               \
                      [list checkbutton "Mute Sound"          "" "" ""        \
                            -variable sound::options(mute)]                   \
                ]                                                             \
            ]                                                                 \
            [list command "Customize..."                      "" "" ""        \
                            -command "custom::open_window Tkabber"]           \
            separator                                                         \
            [list command E&xit                               "" "" "Ctrl Q"  \
                              -command quit]                                  \
      ]                                                                       \
                                                                              \
&Services "" "" 1                                                             \
      [list                                                                   \
            [list command "New Blank &Message"                "" "" ""        \
                              -command message::send_dialog]                  \
            separator                                                         \
            [list command "&Add Contact..."                   "" "" "Ctrl U"  \
                              -command message::send_subscribe_dialog]        \
            [list command "Add &Group..."                     "" "" ""        \
                              -command add_group_dialog]                      \
            [list command "Browse Agents/IM Systems..."       "" "" "Ctrl B"  \
                              -command browser::open]                         \
            [list command "Discover..."                       "" "" ""        \
                              -command disco::browser::open_win]              \
            [list command "Show user info..."                 "" "" ""        \
                            -command userinfo::show_info_dialog]              \
            separator                                                         \
	    [list command "Add group by regexp on JIDs..."    "" "" ""        \
			    -command roster::add_group_by_jid_regexp_dialog]  \
	    [list command "Export roster..."                  "" "" ""        \
			    -command roster::export_to_file]                  \
	    [list command "Import roster..."                  "" "" ""        \
			    -command roster::import_from_file]                \
            separator                                                         \
            [list command "Join Group &Chat..."               "" "" "Ctrl C"  \
                              -command join_group_dialog]                     \
            [list command "Jabber Message &Rules"             "" "" ""        \
                              -command filters::open]                         \
            [list command "Privacy rules..."                  "" "" ""        \
                            -command privacy::request_lists]                  \
      ]                                                                       \
                                                                              \
&Help "" "" 1                                                                 \
      [list                                                                   \
            [list command "&Quick Help..."                "" "" ""            \
                              -command {
help_window "Quick Help" "
Tabs:
    C-F4                Close tab
    C-Left/Right        Move tab left/right
    A-Left/Right        Go to previous/next tab
    A-N         Show tab #N (1-10)
    C-r         Hide/Show roster 

Chats:
    TAB         Complete nickname
    C-Up/Dn     Go to previous/next history message
    A-e         Show emoticons
    C-z         Undo
    C-Z         Redo
    A-PgUp/Dn   Scroll chat window up/down
    Right-click Correct word
"}]                                                                           \
            [list command "&About..."                     "" "" ""            \
                              -command about_window]                          \
            [list cascad   &Debug                            "" debug 1       \
                [debug_buttons]                                               \
            ]                                                                 \
      ]                                                                       \
]
}

proc debug_buttons {} {
    global debug_modules debug_lvls

    set buttons {}
    foreach l [list avatar       \
                    browser      \
                    chat         \
                    conference   \
                    disco        \
                    emoticons    \
                    filetransfer \
                    filters      \
                    hook         \
                    iface        \
                    iq           \
                    jidlink      \
                    jlib         \
                    logger       \
                    login        \
                    message      \
                    negotiate    \
                    nick         \
                    plugins      \
                    presence     \
                    register     \
                    roster       \
                    search       \
                    ssj          \
                    tkabber      \
                    userinfo     \
                    utils        \
                    warning      \
                    zerobot] {
        if {[lsearch -exact $debug_lvls $l] >= 0} {
            set debug_modules($l) 1
        } else {
            set debug_modules($l) 0
        }

        lappend buttons \
                [list checkbutton $l {} {} {} \
                      -variable debug_modules($l) -command debug_update]
    }

    return $buttons
}

proc debug_update {} {
    global debug_modules debug_lvls
    
    set debug_lvls {}
    foreach {k v} [array get debug_modules] {
        if {$v} {
            lappend debug_lvls $k
        }
    }
}


namespace eval jbot {
    proc open_chat_post {chatid type} {
        global usetabbar

        if {[cequal [chat::get_nick $chatid $type]/$type jbot/chat]} {
            set cw [chat::winid $chatid]
            pack forget $cw.csw
            pack $cw.csw -expand yes -fill both -side left
            destroy $cw.status $cw.input $cw.pw0
            if {$usetabbar} {
                .nb itemconfigure [crange [win_id tab $cw] 1 end] \
                    -text jbot \
                    -raisecmd "tab_set_updated [list $cw]
                               hook::run raise_chat_tab_hook [list $cw] [list $chatid]"
            }

            return stop
        }
    }

    proc draw_message {chatid from type body x} {
        if {[cequal [chat::get_nick $from $type]/$type jbot/chat]} {
            set cw [chat::chat_win $chatid]
            foreach line [split [string trimright $body] "\n"] {
                if {[cequal [string index $line 15] " "]} {
                    $cw insert end [string range $line 0 15] me
                    set line [string range $line 16 end]
                    if {([set d [string first " " $line]] > 0) \
                            && (([string first ": " $line] > $d)
                                    || ([string first "last message repeated" \
                                                $line] > $d))} {
                        $cw insert end [string range $line 0 $d] they
                        set line [string range $line [expr $d+1] end]
                    }
                }
                if {[set d [string first ": " $line]] > 0} {
                    $cw insert end [string range $line 0 $d] me
                    set line [string range $line [expr $d+1] end]
                }
                $cw insert end "$line\n" ""
            }

            return stop
        }
    }

    proc normalize_chat_id {chatidvar type} {
        upvar 2 $chatidvar chatid

        if {[cequal [chat::get_nick $chatid $type]/$type jbot/chat]} {
            set chatid [node_and_server_from_jid $chatid]/syslog
        }
    }

    hook::add open_chat_post_hook    [namespace current]::open_chat_post     1
    hook::add draw_message_hook      [namespace current]::draw_message       1
    hook::add normalize_chat_id_hook [namespace current]::normalize_chat_id  1
}


if {(![file exists [set file .jsendrc.tcl]]) \
        && (![file exists [set file ~/.jsendrc.tcl]])} {
    return
}
source $file

namespace eval zerobot {
    proc examine_message {from id type subject body err thread priority x} {
        global userstatus

        switch -- $userstatus {
            away
                -
            xa {
                if {([string length [set body [checkP $from $type $subject \
                                                     $body $x]]] > 0) \
                        && ([catch { notify $from $body } result])} {
                    debugmsg zerobot "notify: $result"
                }
            }

            default {
                reset
            }
        }
    }

    hook::add examine_message_hook   [namespace current]::examine_message   10
}
