# $Id: chats.tcl,v 1.79 2003/06/04 07:22:46 aleksey Exp $

package require textutil

option add *Chat.theyforeground        red3   widgetDefault
option add *Chat.meforeground          blue   widgetDefault
option add *Chat.serverlabelforeground green  widgetDefault
option add *Chat.serverforeground      violet widgetDefault
option add *Chat.errforeground         red    widgetDefault
option add *urlforeground              blue   widgetDefault
option add *urlactiveforeground        red    widgetDefault
option add *Chat.inputheight           3      widgetDefault


namespace eval chat {
    array set opened {}
    set chats(groupchats) {}
    set urlid 0
    set enrichid 0
    custom::defgroup Chat [::msgcat::mc "Chat options."] -group Tkabber
    custom::defvar options(smart_scroll) 0 \
	    [::msgcat::mc "Enable chat window autoscroll only when last message is shown."] -type boolean -group Chat
    custom::defvar options(stop_scroll) 0 [::msgcat::mc "Stop chat window autoscroll."] -type boolean -group Chat
    custom::defvar options(emphasize) 1 \
	[::msgcat::mc "Enable messages emphasize."] \
	-type boolean -group Chat
    set options(default_message_type) chat
    set url_regexp {^(http://|https://|www\.|ftp://|ftp\.)}
}

set chat_width 50
set chat_height 18

plugins::load [file join plugins chat]

proc chat::get_nick {from type} {
    variable chats

    set nick $from
    switch -- $type {
	chat {
	    #regexp {(.*)@.*} $from temp nick
	    set groupid [node_and_server_from_jid $from]
	    if {[lsearch -exact $chats(groupchats) $groupid] >= 0} {
		if {[conference::is_v2 $groupid]} {
		    set nick [conference::get_nick $from]
		} else {
		    set nick [resource_from_jid $from]
		}
	    } else {
		set nick [roster::itemconfig [roster::find_jid $from] -name]
		if {$nick == ""} {
		    set nick [node_from_jid $from]
		}
	    }
	}
	groupchat {
	    set groupid [node_and_server_from_jid $from]
	    if {[conference::is_v2 $groupid]} {
	        set nick [conference::get_nick $from]
	    } else {
		set nick [resource_from_jid $from]
	    }
	}
	default {
	}
    }
    return $nick
}

proc chat::winid {name} {
    global w

    #regsub -all \\. $name | allowed_name 
    set allowed_name [jid_to_tag $name]
    return $w.chat_$allowed_name
}

proc client:message {from id type subject body err thread priority x} {
    debugmsg chat "MESSAGE: $from; $id $type; $subject; $body; $err;\
$thread; $priority; $x"

    if {![cequal [info commands ::ssj::encrypted:input] ""]} {
	set xs {}

	foreach xe $x {
	    jlib::wrapper:splitxml $xe tag vars isempty chdata children

	    if {![cequal [jlib::wrapper:getattr $vars xmlns] \
			 jabber:x:encrypted]} {
		lappend xs $xe
		continue
	    } elseif {[cequal $chdata ""]} {
# in case the sender didn't check the exit code from gpg...
	    } elseif {[catch { ssj::encrypted:input $from $chdata } body]} {
		set body [format [::msgcat::mc ">>> Unable to decipher data: %s <<<"] $body]
	    } else {
		lappend xs $xe
	    }
	}

	set x $xs
    }

    chat::process_message \
	$from $id $type $subject $body $err $thread $priority $x
}

proc chat::process_message {from id type subject body err thread priority x} {
    global font
    variable opened
    variable chats
    variable options

    if {$type == ""} {
	set type $options(default_message_type)
    }

    set chatid [set from [tolower_node_and_domain $from]]
    hook::run normalize_chat_id_hook chatid $type
    hook::run examine_message_hook $from $id $type $subject $body $err \
				   $thread $priority $x

    switch -- $type {
	normal {
	    message::show_dialog $from $type $subject $body $thread $priority $x
	    return
	}
	headline {
	    message::show_headline $from $type $subject $body $thread $priority $x
	    return
	}
	chat {
	    if {$thread != ""} {
		set chats(thread,$chatid) $thread
	    }
	    set chats(id,$chatid) $id
	}
	groupchat {
	    #regexp {(.*)/.*} $from temp chatid
	    set chatid [node_and_server_from_jid $from]

	    if {![is_groupchat $chatid]} return

	    if {![cequal $subject ""]} {
		set_subject $chatid $subject
	    }
	}
	error {
	    set body [error_to_string $err]
	}
	default {
	    debugmsg chat "MESSAGE: UNSUPPORTED message type '$type'"
	}
    }

    chat::open_window $chatid $type

    chat::add_message $chatid $from $type $body $x
    if {[llength $x] > 0} {
        message::show_dialog $from $type $subject $body $thread $priority $x 0
    }
}

proc chat::window_titles {chatid} {
    variable chats

    set chatname [roster::itemconfig [roster::find_jid $chatid] -name]
    if {$chatname != ""} {
	set tabtitlename $chatname
	set titlename $chatname
    } else {
	set titlename $chatid
	if {[lsearch -exact $chats(groupchats) $chatid] >= 0} {
	    set tabtitlename [node_from_jid $chatid]
	} else {
	    set tabtitlename [get_nick $chatid chat]
	}
	if {$tabtitlename == ""} {
	    set tabtitlename $titlename
	}
    }
    return [list $tabtitlename [format [::msgcat::mc "Chat with %s"] $titlename]]
}

proc chat::open_window {chatid type} {
    global font chat_width chat_height loginconf rostericon
    variable opened
    variable chats

    set chatid [tolower_node_and_domain $chatid]
    
    set cw [winid $chatid]

    if {[winfo exists $cw]} {
	if {!$::usetabbar && \
		[info exists ::raise_on_activity] && $::raise_on_activity} {
	    if {$type == "chat"} {
		wm deiconify $cw
	    }
	    raise $cw
	}
	return
    }

    hook::run open_chat_pre_hook $chatid $type

    set chats(type,$chatid) $type
    set chats(ourjid,$chatid) [get_our_jid $chatid $type]
    debugmsg chat "OURJID: $chats(ourjid,$chatid)"
    lappend chats(opened) $chatid
    
    set opened($chatid) $cw
    #debugmsg chat $cw
    #debugmsg chat [array names opened_chats]
    #debugmsg chat $opened_chats($chatid)

    lassign [chat::window_titles $chatid] tabtitlename titlename

    add_win $cw -title $titlename \
	-tabtitle $tabtitlename \
	-class Chat -type $type \
	-raisecmd "focus [list $cw.input]
		   tab_set_updated [list $cw]
                   hook::run raise_chat_tab_hook [list $cw] [list $chatid]"

    frame $cw.status
    pack $cw.status -side top -fill x
    if {[cequal $type chat]} {
	label $cw.status.icon -image $rostericon(user,[get_user_status $chatid])
	pack $cw.status.icon -side left

    }
    if {![cequal [info commands ::ssj::encrypt:toggleP] ""]} {
        if {[cequal [set jid [roster::find_jid $chatid]] ""]} {
            set jid $chatid
        }
        Button $cw.status.encrypted -relief flat \
            -image [message::encrypted:icon $jid] \
            -helptype balloon \
            -helptext [::msgcat::mc "Toggle encryption"] \
            -command [list ssj::encrypt:toggleP $jid]
        ssj::encrypted:trace "
            $cw.status.encrypted configure -image \[message::encrypted:icon $jid\]
        " $jid
        pack $cw.status.encrypted -side left
    }
    if {[cequal $type chat]} {
	menubutton $cw.status.mb -text $chatid -font $font \
	    -menu $cw.status.mb.menu
	create_user_menu $cw.status.mb.menu $chatid
	pack $cw.status.mb -side left
    } else {
        menubutton $cw.status.mb -text [::msgcat::mc "Subject:"] \
	    -direction below \
	    -menu $cw.status.mb.menu -relief flat
	create_conference_menu $cw.status.mb.menu $chatid
	pack $cw.status.mb -side left

	entry $cw.status.subject -font $font
	pack $cw.status.subject -side left -fill x -expand yes

	bind $cw.status.subject <Return> \
	    [list chat::change_subject [double% $chatid]]
    }
    foreach tag [bind Menubutton] {
        if {[string first 1 $tag] >= 0} {
	    regsub -all 1 $tag 3 new
	    bind $cw.status.mb $new [bind Menubutton $tag]
	}
    }

    PanedWin $cw.pw0 -side right -pad 0 -width 8
    pack $cw.pw0 -fill both -expand yes

    set upw [$cw.pw0 add -weight 1 -minsize 0]
    set dow [$cw.pw0 add -weight 0 -minsize 0]

    set isw [ScrolledWindow $cw.isw -scrollbar vertical]
    pack $cw.isw -fill both -expand yes -side bottom -in $dow
    textUndoable $cw.input -width $chat_width -height [option get $cw inputheight Chat] -font $font -wrap word
    $isw setwidget $cw.input
    [winfo parent $dow] configure -height [winfo reqheight $cw.input]
    set chats(inputwin,$chatid) $cw.input

    if {[cequal $type groupchat]} {
	lappend chats(groupchats) $chatid

	PanedWin $cw.pw -side bottom -pad 2 -width 8
	pack $cw.pw -fill both -expand yes -in $upw

	set cf [$cw.pw add -weight 1 -minsize 0]

	set uw [$cw.pw add -weight 0 -minsize 0]
	set chats(userswin,$chatid) $uw.users

	set rosterwidth [option get . chatRosterWidth [winfo class .]]
	if {$rosterwidth == ""} {
	    set rosterwidth [winfo pixels . 3c]
	}

	roster::create $uw.users -width $rosterwidth \
	    -popup roster::groupchat_popup_menu
	pack $uw.users -fill both -side right -expand yes
	[winfo parent $uw] configure -width $rosterwidth
	
	global grouproster
	
	set grouproster(users,$chatid) {}

	roster::addline $uw.users group Users users

	client:presence $chatid "" "" {}

	set pack_in $cf
    } else {
	set cf $cw
	set pack_in $upw
    }

    set chats(chatwin,$chatid) $cf.chat

    set csw [ScrolledWindow $cf.csw -scrollbar vertical]
    pack $csw -expand yes -fill both -side left -in $pack_in
    text $cf.chat -width $chat_width -height $chat_height -font $font -wrap word
    $csw setwidget $cf.chat

    $cf.chat tag configure they \
	-foreground [option get $cw theyforeground Chat]
    $cf.chat tag configure me \
	-foreground [option get $cw meforeground Chat]
    $cf.chat tag configure server_lab \
	-foreground [option get $cw serverlabelforeground Chat]
    $cf.chat tag configure server \
	-foreground [option get $cw serverforeground Chat]
    $cf.chat tag configure err \
	-foreground [option get $cw errforeground Chat]

    set font_bold [eval font create [font actual $font]]
    set font_slanted [eval font create [font actual $font]]
    set font_bold_slanted [eval font create [font actual $font]]
    font configure $font_bold -weight bold
    font configure $font_slanted -slant italic
    font configure $font_bold_slanted -weight bold -slant italic
    $cf.chat tag configure bold -font $font_bold
    $cf.chat tag configure slanted -font $font_slanted
    $cf.chat tag configure bold_slanted -font $font_bold_slanted
    $cf.chat tag configure underlined -underline 1


    $cf.chat configure -state disabled
    focus $cw.input

    bind $cw.input <Shift-Key-Return> { }

    bind $cw.input <Key-Return> [double% "
        chat::send_message [list $cw] [list $chatid] \
	    [list $loginconf(user)] [list $type]
	break"]

    bind $cw.input <Control-z> { %W undo }
    bind $cw.input <Control-Z> { %W redo }

    regsub -all %W [bind Text <Prior>] $cf.chat prior_binding
    regsub -all %W [bind Text <Next>] $cf.chat next_binding
    bind $cw.input <Meta-Prior> $prior_binding
    bind $cw.input <Meta-Next> $next_binding
    bind $cw.input <Alt-Prior> $prior_binding
    bind $cw.input <Alt-Next> $next_binding

    bind $cw <Destroy> [list chat::close_window [double% $chatid]]

    hook::run open_chat_post_hook $chatid $type
}


proc chat::close_window {chatid} {
    global chat_width chat_height
    variable opened
    variable chats

    set cw [winid $chatid]

    if {[info exists opened($chatid)]} {
	unset opened($chatid)
	set idx [lsearch -exact $chats(opened) $chatid]
	set chats(opened) [lreplace $chats(opened) $idx $idx]
	debugmsg chat $chats(opened)

	if {$chats(type,$chatid) == "groupchat"} {
	    jlib::send_presence \
		-to $chatid/[get_our_groupchat_nick $chatid] \
		-type unavailable
	    client:presence $chatid unavailable "" {}
	    foreach jid [get_jids_of_user $chatid] {
		client:presence $jid unavailable "" {}
	    }

	    if {[conference::is_v2 $chatid]} {
		conference::exit $chatid
	    }
	}

	destroy $cw
    }
}

proc chat::users_win {chatid} {
    variable chats
    return $chats(userswin,$chatid)
}

proc chat::chat_win {chatid} {
    variable chats
    return $chats(chatwin,$chatid)
}

proc chat::input_win {chatid} {
    variable chats
    return $chats(inputwin,$chatid)
}

proc chat::is_groupchat {chatid} {
    variable chats
    return [lcontain $chats(groupchats) $chatid]
}

proc chat::create_user_menu {path jid} {
    set m [menu $path -tearoff 0]

    $m add command -label [::msgcat::mc "Invite to conference..."] \
	-command [list chat::invite_dialog $jid]
    $m add command -label [::msgcat::mc "Add user..."] \
	-command [list message::send_subscribe_dialog \
		      [node_and_server_from_jid $jid]]
    hook::run chat_create_user_menu_hook $path $jid
    $m add separator
    $m add command -label [::msgcat::mc "Send users..."] \
	-command [list roster::send_users_dialog $jid]
    $m add command -label [::msgcat::mc "Send file..."] \
	-command [list ft::send_file_dialog $jid]
    $m add command -label [::msgcat::mc "Send file via Jidlink..."] \
	-command [list ftjl::send_file_dialog $jid]
    $m add separator
    $m add command -label [::msgcat::mc "Show info..."] \
	-command [list userinfo::open $jid]
    $m add command -label [::msgcat::mc "Show history..."] \
	-command [list logger::show_log $jid]
    return $path
}

proc chat::create_conference_menu {path jid} {
    set m [menu $path -tearoff 0]
    $m add command -label [::msgcat::mc "Invite users..."] \
	-command [list chat::invite_dialog2 $jid]
    $m add command -label [::msgcat::mc "Show history..."] \
	-command [list logger::show_log $jid]
    add_conference_custom_presence_menu $m $jid
    hook::run chat_create_conference_menu_hook $path $jid
    return $path
}

proc chat::add_conference_custom_presence_menu {m jid} {
    set mm [menu $m.custom_presence -tearoff 0]

    $mm add command -label [::msgcat::mc "Online"] \
	-command [list send_custom_presence $jid available]
    $mm add command -label [::msgcat::mc "Free to chat"] \
	-command [list send_custom_presence $jid chat]
    $mm add command -label [::msgcat::mc "Away"] \
	-command [list send_custom_presence $jid away]
    $mm add command -label [::msgcat::mc "Extended Away"] \
	-command [list send_custom_presence $jid xa]
    $mm add command -label [::msgcat::mc "Do not disturb"] \
	-command [list send_custom_presence $jid dnd]
    #$mm add command -label [::msgcat::mc "Offline"] \
    #    -command [list send_custom_presence $jid unavailable]

    $m add cascad -label [::msgcat::mc "Send custom presence"] -menu $mm
}



proc chat::send_message {cw chatid user type} {
    set iw $cw.input

    set body [$iw get 1.0 "end -1 chars"]

    debugmsg chat "SEND_MESSAGE:\
 [list $chatid] [list $user] [list $body] [list $type]"

    set chatw [chat_win $chatid]
    $chatw mark set start_message "end -1 chars"
    $chatw mark gravity start_message left

    hook::run chat_send_message_hook $chatid $user $body $type

    $iw delete 1.0 end
}

proc chat::add_message {chatid from type body x} {
    variable chats
    variable options

    set chatw [chat_win $chatid]
    $chatw configure -state normal

    set scroll [expr {([lindex [$chatw yview] 1] == 1) ? 1 : 0}]

    hook::run draw_message_hook $chatid $from $type $body $x

    if {![$chatw compare "end -1 chars linestart" == "end -1 chars"]} {
	$chatw insert end "\n"
    }

    if {!$options(stop_scroll) && \
	    (!$options(smart_scroll) || ($options(smart_scroll) && $scroll) ||
	    [chat::is_our_jid $chatid $from])} {
	$chatw see end
    }

    $chatw configure -state disabled

    #set cw [winid $chatid]
    #tab_set_updated $cw 1 mesg_to_user
    #update
}

proc chat::highlighttext {chatw tag color cursor} {
	$chatw configure -cursor $cursor
	$chatw tag configure $tag -foreground $color
}

proc chat::add_emoteiconed_text {chatw body defaulttag} {
    variable url_regexp
    variable options

    set num_of_emoticons 0
    set words [textutil::splitx $body {([\t \r\n]+)}]

    foreach word $words {
	if {$num_of_emoticons < 200} {
	    set image [emoteicons::get $word]
	    if {$image != ""} {
		$chatw image create end -image $image
		incr num_of_emoticons
		continue
	    }
	}
	if {[regexp -- $url_regexp $word]} {
	    variable urlid
	    set tag url$urlid
	    set urlfg    [option get $chatw urlforeground       Text]
	    set urlactfg [option get $chatw urlactiveforeground Text]
	    $chatw tag configure $tag -foreground $urlfg -underline 1
	    $chatw tag bind $tag <1> [list browseurl [double% $word]]
	    $chatw tag bind $tag <Any-Enter> \
		[list chat::highlighttext $chatw $tag $urlactfg hand2]
	    $chatw tag bind $tag <Any-Leave> \
		[list chat::highlighttext $chatw $tag $urlfg xterm]
	    
	    $chatw insert end $word $tag
	    incr urlid
	} else {
	    if {!$options(emphasize)} {
		$chatw insert end $word $defaulttag
	    } else {
		set enrichedText(bold) 0
		set enrichedText(italic) 0
		set enrichedText(underline) 0

		set tmp $word
		while {[regexp -- "^(_|/|\\*)(.+)(_|/|\\*)$" $tmp wholeMatch e1 subWord e2]} {
		    if { $e1 != $e2 } {
			break
		    }
		    if { "*" == $e1 } {
			set enrichedText(bold) 1
		    } elseif { "/" == $e1 } {
			set enrichedText(italic) 1
		    } elseif { "_" == $e1 } {
			set enrichedText(underline) 1
		    }
		    set tmp $subWord
		}

		set tag $defaulttag
		switch -- $enrichedText(bold),$enrichedText(italic) {
		    1,1 {lappend tag bold_slanted}
		    1,0 {lappend tag bold}
		    0,1 {lappend tag slanted}
		}
		if {$enrichedText(underline) == 1} {
		    lappend tag underlined
		}
		$chatw insert end $tmp $tag
	    }
	}
    }
}



proc chat::open_to_user {user} {
    set jid [get_jid_of_user $user]

    if {[cequal $jid ""]} {
	set jid $user
    }

    set cw [winid $jid]
    if {[winfo exists $cw]} {
	focus -force $cw.input
	return
    }

    chat::open_window $jid chat
}


proc chat::change_presence {jid status} {
    global grouproster
    global rostericon
    variable chats
    variable opened

    set group [node_and_server_from_jid $jid]
    set nick [get_nick $jid groupchat]

    if {[resource_from_jid $jid] == ""} {
	return
    }

    if {[info exists opened($group)]} {
	if {[lcontain $chats(groupchats) $group]} {
	    debugmsg chat "ST: $jid $status"
	    if {[cequal $status unavailable] || [cequal $status error]} {
		if {$status == "error" && [is_our_jid $group $jid]} {
		    add_message $group $group error \
			[format [::msgcat::mc "Error %s"] [get_jid_presence_info status $jid]] {}
		}
		if {$status == "unavailable" && [is_our_jid $group $jid]} {
		    add_message $group $group error \
			[::msgcat::mc "Disconnected"] {}
		}
		debugmsg chat "$jid UNAVAILABLE"
		muc::process_unavailable $group $nick
		hook::run chat_user_exit $group $nick
		lvarpop grouproster(users,$group) \
		    [lsearch -exact $grouproster(users,$group) $jid]
		debugmsg chat "GR: $grouproster(users,$group)"
		chat::redraw_roster_after_idle $group
	    } else {
		set userswin [users_win $group]
		if {![lcontain $grouproster(users,$group) $jid]} {
		    #roster::addline $userswin jid $nick $jid
		    lappend grouproster(users,$group) $jid
		    set grouproster(status,$group,$jid) $status

		    muc::process_available $group $nick
		    hook::run chat_user_enter $group $nick
		}
		set grouproster(status,$group,$jid) $status
		roster::changeicon $userswin $jid $rostericon(user,$status)
		roster::changeforeground $userswin $jid $status
		chat::redraw_roster_after_idle $group
	    }
	}

	#roster:changeicon $w.users [user_from_jid $from] $status
    }

    set cw [winid $jid]

    if {[winfo exists $cw.status.icon]} {
	$cw.status.icon configure -image $rostericon(user,$status)
    }

    set user [node_and_server_from_jid $jid]
    set cw [winid $user]

    if {[winfo exists $cw.status.icon]} {
	$cw.status.icon configure \
	    -image $rostericon(user,[get_user_status $user])
    }
}

proc chat::redraw_roster {group} {
    global grouproster
    global rostericon

    set userswin [users_win $group]

    set grouproster(users,$group) [lsort $grouproster(users,$group)]
    roster::clear $userswin 0
    #roster::addline $userswin group Users users

    set levels {}
    foreach jid $grouproster(users,$group) {
	if {[info exists ::muc::users(role,$jid)]} {
	    if {$::muc::users(role,$jid) == "" && \
		    $::muc::users(affiliation,$jid) == ""} {
		set level "aGreen mans"
		lappend levels $level
		lappend levelusers($level) $jid
	    } else {
		if {$::muc::users(role,$jid) != "" && \
			$::muc::users(role,$jid) != "none"} {
		    set level $::muc::users(role,$jid)
		    switch -- $level {
			moderator   {set level 4[::msgcat::mc "Moderators"]}
			participant {set level 5[::msgcat::mc "Participants"]}
			visitor     {set level 6[::msgcat::mc "Visitors"]}
		    }
		    lappend levels $level
		    lappend levelusers($level) $jid
		}

		#if {$::muc::users(affiliation,$jid) != "" && \
		#	$::muc::users(affiliation,$jid) != "none"} {
		#    set level $::muc::users(affiliation,$jid)
		#    switch -- $level {
		#	owner   {set level 0Owners}
		#	admin   {set level 1Admins}
		#	member  {set level 2Members}
		#	outcast {set level 9Outcasts}
		#    }
		#    lappend levels $level
		#    lappend levelusers($level) $jid
		#}
	    }
	} else {
	    set level 2[::msgcat::mc "Users"]
	    lappend levels $level
	    lappend levelusers($level) $jid
	}	    
    }
    set levels [lrmdups $levels]

    foreach level $levels {
	roster::addline $userswin group \
	    "[crange $level 1 end] ([llength $levelusers($level)])" \
	    [crange $level 1 end]
	set jid_nicks {}
	foreach jid $levelusers($level) {
	    lappend jid_nicks [list $jid [get_nick $jid groupchat]]
	}
	set jid_nicks [lsort -index 1 -dictionary $jid_nicks]

	foreach item $jid_nicks {
	    lassign $item jid nick
	    set status $grouproster(status,$group,$jid)

	    roster::addline $userswin jid $nick $jid
	    roster::changeicon $userswin $jid $rostericon(user,$status)
	    roster::changeforeground $userswin $jid $status
	}
    }

    roster::update_scrollregion $userswin
}

proc chat::redraw_roster_after_idle {group} {
    variable afterid

    if {[info exists afterid($group)]} \
	return

    set afterid($group) [after idle "
	chat::redraw_roster [list $group]
	unset [list chat::afterid($group)]
    "]
}


proc chat::get_our_jid {chatid type} {
    variable chats

    switch -- $type {
	groupchat {
	    return $chatid/[get_our_groupchat_nick $chatid]
	}
	chat {
	    set groupid [node_and_server_from_jid $chatid]
	    if {[lsearch -exact $chats(groupchats) $groupid] >= 0} {
		return $groupid/[get_our_groupchat_nick $groupid]
	    } else {
		global loginconf
		return \
		    ${loginconf(user)}@$loginconf(server)/$loginconf(resource)
	    }
	}
    }
    return ""
}


proc chat::set_subject {chatid subject} {
    variable opened

    set cw [winid $chatid]

    if {[info exists opened($chatid)]} {
	$cw.status.subject delete 0 end
	$cw.status.subject insert 0 $subject
    }
}

proc chat::change_subject {chatid} {
    set cw [winid $chatid]

    set subject [$cw.status.subject get]

    # Probably it's not a good idea to translate this message.
    message::send_msg $chatid -type groupchat -subject $subject \
		-body "/me has changed the subject to: $subject"
}


proc chat::is_our_jid {chatid jid} {
    variable chats
    return [cequal $chats(ourjid,$chatid) $jid]
}

proc chat::our_jid {chatid} {
    variable chats
    return $chats(ourjid,$chatid)
}

proc chat::invite_dialog {user} {
    variable chats
    variable opened
    global invite_gc

    set jid [get_jid_of_user $user]

    if {[cequal $jid ""]} {
        set jid $user
    }

    set gw .invite
    catch { destroy $gw }

    if {[catch { set nick [roster::get_label $user] }]} {
	if {[catch {set nick [roster::get_label \
				  [node_and_server_from_jid $user]] }]} {
	    if {[catch { set nick [chat::get_nick $user groupchat] }]} {
		set nick $user
	    }
	}
    }

    Dialog $gw -title [format [::msgcat::mc "Invite %s to conferences"] $nick] \
	-separator 1 -anchor e -default 0 -cancel 1

    set gf [$gw getframe]

    set sw [ScrolledWindow $gf.sw]
    set sf [ScrollableFrame $sw.sf]
    pack $sw -expand yes -fill both
    $sw setwidget $sf
    set sff [$sf getframe]

    bindscroll $sf

    $gw add -text [::msgcat::mc "Invite"] \
	-command [list chat::invitation $gw $jid 0]
    $gw add -text [::msgcat::mc "Cancel"] -command "destroy $gw"

    catch { unset invite_gc }

    set i 0
    foreach gc [lsort [array names opened]] {
        if {[lsearch -exact $chats(groupchats) $gc] >= 0} {
            set title [node_from_jid $gc]

            set cb [checkbutton $sff.$i -variable invite_gc($gc) -text $title]
            bind $cb <Any-Enter>  [list balloon::default_balloon $cb enter \
				       %X %Y $gc]
            bind $cb <Any-Motion> [list balloon::default_balloon $cb motion \
				       %X %Y $gc]
            bind $cb <Any-Leave>  [list balloon::default_balloon $cb leave \
				       %X %Y]
            pack $cb -anchor w
            incr i
        }
    }
    if {$i == 0} {
        MessageDlg ${gw}_err -aspect 50000 -icon info \
	    -message [::msgcat::mc "No conferences in progress..."] -type user \
	    -buttons ok -default 0 -cancel 0
        return
    }

    $gw draw
}

proc chat::invite_dialog2 {chatid} {
    variable chats
    variable opened
    global invite_gc

    set gw .invite
    catch { destroy $gw }

    if {[lsearch -exact $chats(groupchats) $chatid] >= 0} {
        set title [node_from_jid $chatid]
    } else {
        set title [get_nick $chatid chat]
    }

    Dialog $gw -title [format [::msgcat::mc "Invite users to %s"] $title] \
	-separator 1 -anchor e -default 0 -cancel 1

    set gf [$gw getframe]

    set sw [ScrolledWindow $gf.sw]
    set sf [ScrollableFrame $sw.sf]
    pack $sw -expand yes -fill both
    $sw setwidget $sf
    set sff [$sf getframe]

    bindscroll $sf

    $gw add -text [::msgcat::mc "Invite"] \
	-command [list chat::invitation $gw $chatid 1]
    $gw add -text [::msgcat::mc "Cancel"] -command "destroy $gw"

    catch { unset invite_gc }

    set choices {}
    foreach choice $roster::roster(jids) {
	if {![cequal $roster::roster(category,$choice) conference]} {
	    lappend choices [list $choice [roster::get_label $choice]]
	}
    }

    set i 0
    foreach choice [lsort -index 1 $choices] {
	set gc [lindex $choice 0]

	set cb [checkbutton $sff.$i -variable invite_gc($gc) \
		    -text [lindex $choice 1]]
	bind $cb <Any-Enter>  [list balloon::default_balloon $cb enter  %X %Y \
                                    $gc]
	bind $cb <Any-Motion> [list balloon::default_balloon $cb motion %X %Y \
                                    $gc]
	bind $cb <Any-Leave>  [list balloon::default_balloon $cb leave  %X %Y]
        pack $cb -anchor w
        incr i
    }
    if {$i == 0} {
        MessageDlg ${gw}_err -aspect 50000 -icon info \
	    -message [::msgcat::mc "No users in roster..."] -type user \
	    -buttons ok -default 0 -cancel 0
        return
    }

    $gw draw
}

proc chat::invitation {gw jid {usersP 0}} {
    global invite_gc

    set sf [$gw getframe].sw.sf
    set choices {}
    foreach gc [array names invite_gc] {
        if {$invite_gc($gc)} {
            lappend choices $gc
        }
    }

    destroy $gw

    foreach choice $choices {
        if {$usersP} {
            set to $choice
            set chatid $jid
        } else {            
            set to $jid
            set chatid $choice
        }

        message::send_msg $to -type normal \
	    -body [format [::msgcat::mc "Please join %s"] $chatid] \
	    -xlist [list [jlib::wrapper:createtag x \
			      -vars [list xmlns jabber:x:conference \
					 jid   $chatid]]]
    }
}


