# $Id: emoticons.tcl,v 1.11 2002/11/01 20:34:10 alexey Exp $

namespace eval emoteicons {
    array set emoteicons {}

    variable lasttext ""
    variable lastX
    variable lastY
}

proc emoteicons::add {word image} {
    variable emoteicons
    set emoteicons($word) $image
}

proc emoteicons::get {word} {
    variable emoteicons

    if {[info exists emoteicons($word)]} {
	return $emoteicons($word)
    } else {
	return ""
    }
}


#image create photo facestartled -file [fullpath gnus-smilies/FaceStartled.gif]
#image create photo facestraight -file [fullpath gnus-smilies/FaceStraight.gif]
#image create photo faceironic   -file [fullpath gnus-smilies/FaceIronic.gif]
#image create photo facehappy    -file [fullpath jajc-bitmaps/facehappy.gif]
#image create photo facesad      -file [fullpath jajc-bitmaps/facesad.gif]
#image create photo facegrinning -file [fullpath jajc-bitmaps/facegrinning.gif]
#image create photo faceyukky    -file [fullpath jajc-bitmaps/faceyukky.gif]
#image create photo facewinking  -file [fullpath jajc-bitmaps/facewinking.gif]
#image create photo beer         -file [fullpath jajc-bitmaps/beer.gif]
#
#
#
#emoteicons::add :)     facehappy
#emoteicons::add :-)    facehappy
#emoteicons::add :(     facesad
#emoteicons::add :-(    facesad
#emoteicons::add :-O    facestartled
#emoteicons::add :-|    facestraight
#emoteicons::add :-/    faceironic
#emoteicons::add :-D    facegrinning
#emoteicons::add ";)"   facewinking
#emoteicons::add ";-)"  facewinking
#emoteicons::add :-P    faceyukky
#emoteicons::add :P     faceyukky
#emoteicons::add :beer: beer
#emoteicons::add (B)    beer


proc emoteicons::load_dir {dir} {
    set icondef_path [file join $dir icondef.xml]
    if {![file isfile $icondef_path]} {
	### TODO: some error messages
	return
    }
    set f [open $icondef_path]
    set icondef [read $f]
    close $f

    set parser [jlib::wrapper:new "#" "#" \
		    [list emoteicons::parse_icondef $dir]]
    jlib::wrapper:elementstart $parser stream:stream {} {}
    jlib::wrapper:parser $parser parse $icondef
    jlib::wrapper:parser $parser configure -final 0
}

proc emoteicons::parse_icondef {dir xmldata} {
    jlib::wrapper:splitxml $xmldata tag vars isempty chdata children

    if {$tag != "icondef"} {
	# TODO: error message
	return
    }

    foreach child $children {
	parse_item $dir $child
    }

}

proc emoteicons::parse_item {dir item} {
    jlib::wrapper:splitxml $item tag vars isempty chdata children

    switch -- $tag {
	name {}
	version {}
	description {}
	author {}
	creation {}
	icon {
	    parse_icon $dir $children
	}
    }
}

proc emoteicons::parse_icon {dir items} {
    variable txtdefaults
    variable txtlabels

    set txts {}
    set txtdefault ""
    set graphic ""
    foreach item $items {
	jlib::wrapper:splitxml $item tag vars isempty chdata children
	switch -- $tag {
	    text {
		lappend txts $chdata
		if {$txtdefault == "" || \
			[jlib::wrapper:getattr $vars default] == "true"} {
		    set txtdefault $chdata
		}
	    }
	    graphic {
		switch -- [jlib::wrapper:getattr $vars type] {
		    image/gif {set graphic $chdata}
		}
	    }
	    sound {}
	}
    }
    #debugmsg emoticons "E: $graphic; $txts"
    if {$graphic != ""} {
	set img $dir/$graphic
	image create photo $img -file [file join $dir $graphic]
	foreach txt $txts {
	    emoteicons::add $txt $img
	}
	set txtdefaults($img) $txtdefault
	set txtlabels($img) [file rootname [file tail $graphic]]
    }
}

emoteicons::load_dir [fullpath emoticons-tkabber]



proc emoteicons::show_menu {iw} {
    variable txtdefaults
    variable txtlabels

    set m .emoticonsmenu
    if {[winfo exists $m]} {
	destroy $m
    }
    menu $m -tearoff 0
    set imgs [array names txtdefaults]
    set rows [expr {floor(sqrt([llength $imgs]))}]
    set row 0

    foreach img $imgs {
	if {$row >= $rows} {
	    $m add command -image $img -columnbreak 1 \
		-label $txtlabels($img) \
		-command [list [namespace current]::insert $iw \
						$txtdefaults($img)]
	    set row 1
	} else {
	    $m add command -image $img \
		-label $txtlabels($img) \
		-command [list [namespace current]::insert $iw \
						$txtdefaults($img)]
	    incr row
	}
    }

    bind $m <Any-Enter>  \
	[list [namespace current]::balloon $m enter  %X %Y %x %y]
    bind $m <Any-Motion> \
	[list [namespace current]::balloon $m motion %X %Y %x %y]
    bind $m <Any-Leave>  \
	[list [namespace current]::balloon $m leave  %X %Y %x %y]

    tk_popup $m [winfo pointerx .] [winfo pointery .]
}

# trying to get motion events in a menu is problematic...

proc emoteicons::balloon {w action X Y x y} {
    variable lasttext
    variable lastX
    variable lastY

    if {[cequal [set index [$w index @$x,$y]] none]} {
	if {![cequal $lasttext ""]} {
	    balloon::default_balloon $w leave $lastX $lastY
	}

	return
    }

    set text [$w entrycget $index -label]
    switch -- $action {
        motion {
            if {![cequal $text $lasttext]} {
		if {![cequal $lasttext ""]} {
                    balloon::default_balloon $w leave $lastX $lastY
		}

                balloon::default_balloon $w enter [set lastX $X] \
			[set lastY $Y] [set lasttext $text]
            }
        }

        leave {
            set lasttext ""
        }
    }

    balloon::default_balloon $w $action $X $Y $text
}

proc emoteicons::insert {iw text} {
    set p ""
    switch -- [$iw get "insert - 1 chars"] {
	"" - " " - "\n" {}

	default         { 
	    if {![cequal [$iw index "insert -1 chars"] 1.0]} {
		set p " "
	    }
	}
    }

    $iw insert insert "$p$text "
}

proc emoteicons::setup_bindings {chatid type} {
    set iw [chat::input_win $chatid]

    bind $iw <Meta-e> \
	[list emoteicons::show_menu $iw]
    bind $iw <Meta-e> +break

    bind $iw <Alt-e> \
	[list emoteicons::show_menu $iw]
    bind $iw <Alt-e> +break
}


hook::add open_chat_post_hook emoteicons::setup_bindings
