# $Id: joingrdialog.tcl,v 1.30 2003/06/01 18:57:16 aleksey Exp $

#set gr_nick $user
set gr_group "talks"
if {[info exists loginconf(server)]} {
    set gr_server conference.$loginconf(server)
} else {
    set gr_server conference.jabber.org
}
set gr_passwd ""
set gr_v2 0

custom::defvar gr_nick_list {} \
        [::msgcat::mc "Join group dialog data (nicks)."] -group Hidden
custom::defvar gr_group_list {} \
        [::msgcat::mc "Join group dialog data (groups)."] -group Hidden
custom::defvar gr_server_list {} \
        [::msgcat::mc "Join group dialog data (servers)."] -group Hidden

proc ecursor_entry {e} {

    $e icursor end
    bind $e <FocusIn> [list $e selection range 0 end]
}

proc pack_input {fr row var lab args} {

    label $fr.l$var -text $lab
    ecursor_entry [eval entry $fr.$var -textvar $var $args]

    grid $fr.l$var -row $row -column 0 -sticky e
    grid $fr.$var  -row $row -column 1 -sticky ew
}

proc pack_combo {fr row var lab args} {

    label $fr.l$var -text $lab
    ecursor_entry [eval ComboBox $fr.$var -textvariable $var $args].e

    grid $fr.l$var -row $row -column 0 -sticky e
    grid $fr.$var  -row $row -column 1 -sticky ew
}

proc update_combo_list {list entry num} {

    set ind [lsearch -exact $list $entry]
    if {$ind >= 0} {
	set newlist [linsert [lreplace $list $ind $ind] 0 $entry]
    } else {
	set newlist [linsert $list 0 $entry]
    }
    if {[llength $newlist] > $num} {
	return [lreplace $newlist end end]
    } else {
	return $newlist
    }
}

proc join_group_dialog {args} {
    global gr_nick gr_group gr_server
    global gr_nick_list gr_group_list gr_server_list

    foreach {opt val} $args {
	switch -- $opt {
	    -server {
		set server $val
	    }
	    -group {
		set group $val
	    }
	    -nick {
		set nick $val
	    }
	}
    }

    set gw .joingroup
    catch { destroy $gw }

    Dialog $gw -title [::msgcat::mc "Join group"] -separator 1 -anchor e \
	    -default 0 -cancel 1

    set gf [$gw getframe]
    grid columnconfigure $gf 1 -weight 1

    if {[info exists group]} {
	set gr_group $group
#	set gr_group_list [update_combo_list $gr_group_list $gr_group 20]
    } elseif {[llength $gr_group_list]} {
	set gr_group [lindex $gr_group_list 0]
    }
    if {[info exists server]} {
	set gr_server $server
#	set gr_server_list [update_combo_list $gr_server_list $gr_server 10]
    } elseif {[llength $gr_server_list]} {
	set gr_server [lindex $gr_server_list 0]
    }
    if {[info exists nick]} {
	set gr_nick $nick
#	set gr_nick_list [update_combo_list $gr_nick_list $gr_nick 10]
    } else {
	set gr_nick [get_group_nick ${gr_group}@$gr_server $gr_nick]
    }

    pack_combo $gf 0 gr_nick [::msgcat::mc "Nick:"] -values $gr_nick_list
    pack_combo $gf 1 gr_group [::msgcat::mc "Group:"] -values $gr_group_list
    pack_combo $gf 2 gr_server [::msgcat::mc "Server:"] -values $gr_server_list
    pack_input $gf 3 gr_passwd [::msgcat::mc "Password:"] -width 30 -show *

    checkbutton $gf.v2 -variable gr_v2 -text [::msgcat::mc "use v2 protocol"]
    grid $gf.v2 -row 4 -column 1 -sticky w

    $gw add -text [::msgcat::mc "Join"] -command "join_group1 $gw"
	    
    $gw add -text [::msgcat::mc "Cancel"] -command "destroy $gw"

    $gw draw $gf.gr_nick
}

proc join_group1 {gw} {
    global gr_nick gr_group gr_server
    global gr_nick_list gr_group_list gr_server_list

    destroy $gw

    set gr_nick_list [update_combo_list $gr_nick_list $gr_nick 10]
    set gr_group_list [update_combo_list $gr_group_list $gr_group 20]
    set gr_server_list [update_combo_list $gr_server_list $gr_server 10]
    set custom::saved(::::gr_nick_list) $gr_nick_list
    set custom::saved(::::gr_group_list) $gr_group_list
    set custom::saved(::::gr_server_list) $gr_server_list
    custom::store

    join_group ${gr_group}@$gr_server $gr_nick
}

proc join_group {groupid nick} {
    global gr_v2

    muc::request_negotiation $groupid
    muc::wait_for_negotiation $groupid

    if {!$gr_v2} {
	if {[muc::is_compatible $groupid]} {
	    global gr_passwd
	    muc::join_group $groupid $nick $gr_passwd
	} else {
	    jlib::send_presence -to ${groupid}/${nick}
	    set_our_groupchat_nick $groupid $nick
	    chat::open_window $groupid groupchat
	}
    } else {
	set_our_groupchat_nick $groupid $nick
	conference::join $groupid $nick
    }
}

proc set_our_groupchat_nick {group nick} {
    global groupchats

    set group [tolower_node_and_domain $group]
    set groupchats(nick,$group) $nick
    debugmsg conference [array get groupchats]
}

proc get_our_groupchat_nick {group} {
    global groupchats
    
    debugmsg conference [array get groupchats]
    debugmsg conference "GET NICK: [list $group]"
    return $groupchats(nick,$group)
}

proc add_group_dialog {} {
    global gra_group gra_server

    set gw .addgroup
    catch { destroy $gw }

    Dialog $gw -title [::msgcat::mc "Add group"] -separator 1 -anchor e \
	    -default 0 -cancel 1

    set gf [$gw getframe]

    label $gf.lgroup -text Group:
    ecursor_entry [entry $gf.group -textvariable gra_group]
    label $gf.lserver -text Server:
    ecursor_entry [entry $gf.server -textvariable gra_server]

    grid $gf.lgroup  -row 0 -column 0 -sticky e
    grid $gf.group   -row 0 -column 1 -sticky ew
    grid $gf.lserver -row 1 -column 0 -sticky e
    grid $gf.server  -row 1 -column 1 -sticky ew

    $gw add -text [::msgcat::mc "Add"] -command "add_conference $gw"
    $gw add -text [::msgcat::mc "Cancel"] -command "destroy $gw"

    $gw draw $gf.group
}

proc add_conference {gw} {
    global gra_group gra_server

    destroy $gw

    jlib::send_iq set \
	[jlib::wrapper:createtag query \
	     -vars {xmlns jabber:iq:roster} \
	     -subtags [list [jlib::wrapper:createtag item \
				 -vars [list jid ${gra_group}@$gra_server \
					    category conference] \
				 -subtags [list [jlib::wrapper:createtag \
						     group \
						     -chdata Conferences]]]]]

}


namespace eval conference {}

proc conference::join {conf nick} {
    variable confs


    set confs(v2,$conf) 1
    set confs(nick,$conf) $nick

    jlib::send_presence -to $conf

    jlib::send_iq get \
	[jlib::wrapper:createtag enter \
	     -vars {xmlns jabber:iq:conference}] \
	-to $conf -command [list conference::parse_get_result $conf]
    

}

proc conference::parse_get_result {jid res child} {
    variable confs

    debugmsg conference "$res $child"

    if {![cequal $res OK]} {
	MessageDlg .join_group_error -aspect 50000 -icon error \
	    -message [format [::msgcat::mc "Get conference info failed: %s"] [error_to_string $child]] -type user \
	    -buttons ok -default 0 -cancel 0
	return ""
    }

    set confs(reqfields,$jid) {}

    jlib::wrapper:splitxml $child tag vars isempty chdata children

    if {[cequal [jlib::wrapper:getattr $vars xmlns] jabber:iq:conference]} {
	conference::parse_req_fields $jid $children
    }

    jlib::send_iq set \
	[jlib::wrapper:createtag enter \
	     -vars {xmlns jabber:iq:conference} \
	     -subtags $confs(reqfields,$jid)
	] \
	-to $jid -command [list conference::parse_join_result $jid]
    
}

proc conference::parse_req_fields {jid items} {
    variable confs



    foreach item $items {
	jlib::wrapper:splitxml $item tag vars isempty chdata children

	switch -- $tag {
	    name    {set confs(name,$jid) $chdata}
	    nick    {
		lappend confs(reqfields,$jid) \
		    [jlib::wrapper:createtag nick -chdata $confs(nick,$jid)]
		lappend confs(reqfields,$jid) \
		    [jlib::wrapper:createtag nick -chdata $confs(nick,$jid)_]
		lappend confs(reqfields,$jid) \
		    [jlib::wrapper:createtag nick -chdata $confs(nick,$jid)__]
		lappend confs(reqfields,$jid) \
		    [jlib::wrapper:createtag nick]
		#lappend confs(reqfields,$jid) \
		#    [jlib::wrapper:createtag secret -chdata $confs(nick,$jid)]
		lappend confs(reqfields,$jid) \
		    [jlib::wrapper:createtag privacy]
	    }
	    privacy {
		lappend confs(reqfields,$jid) \
		    [jlib::wrapper:createtag privacy]
}
	    key     {}
	    secret  {
		global gr_passwd
		lappend confs(reqfields,$jid) \
		    [jlib::wrapper:createtag secret -chdata $gr_passwd]
	    }
	    default {debugmsg conference "Unknown iq:conference tag '$tag'"}
	}
    }
}

proc conference::parse_join_result {jid res child} {
    variable confs

    debugmsg conference "JOIN: $res $child"

    if {![cequal $res OK]} {
	MessageDlg .join_group_error -aspect 50000 -icon error \
	    -message [format [::msgcat::mc "Join failed: %s"] [error_to_string $child]] -type user \
	    -buttons ok -default 0 -cancel 0
	return ""
    }

    chat::open_window $jid groupchat

    jlib::wrapper:splitxml $child tag vars isempty chdata children

    if {[cequal [jlib::wrapper:getattr $vars xmlns] jabber:iq:conference]} {
	foreach child $children {
	    jlib::wrapper:splitxml $child tag vars isempty chdata childrens
	    switch -- $tag {
		nick {
		    debugmsg conference "OURNICK: $chdata"
		    set_our_groupchat_nick $jid $chdata
		}
		id {
		    debugmsg conference "OURID: $chdata"
		    set ::chat::chats(ourjid,$jid) $chdata
		}
	    }
	}
    }
}

proc conference::proc_browse {parent jid} {
    global browser::browser
    
    if {[cequal $browser(category,$jid) user]} {
	if {![cequal $browser(type,$jid) remove]} {
	    chat::change_presence $jid available
	    #jlib::send_presence -to help@conference.localhost
	    #-type probe
	} else {
	    debugmsg conference "REMOVE $jid"
	    chat::change_presence $jid unavailable
	}
    }
}

proc conference::is_v2 {conf} {
    variable confs
    return [info exists confs(v2,$conf)]
}

proc conference::get_nick {jid} {
    global browser::browser

    if {![info exists browser(name,$jid)]} {
	return $jid
    } else {
	debugmsg conference "NICK: $jid - $browser(name,$jid)"
	return $browser(name,$jid)
    }
}

proc conference::set_nick {conf nick} {
    variable confs

    jlib::send_iq set \
	[jlib::wrapper:createtag query \
	     -vars [list xmlns jabber:iq:conference] \
	     -subtags [list [jlib::wrapper:createtag nick -chdata $nick]]] \
	-to $conf 
    #-command [list conference::]
}

proc conference::exit {conf} {
    variable confs
    unset confs(v2,$conf)
}

###############################################################################

proc conference::create_room_dialog {} {
    set w .createroom

    if {[winfo exists $w]} {
	#$w draw
	#return
	destroy $w
    }

    Dialog $w -title [::msgcat::mc "Create Room"] -separator 1 -anchor e \
	    -default 0 -cancel 1

    set f [$w getframe]

    pack_input $f 0 grcr(address)     [::msgcat::mc "Address:"]
    pack_input $f 1 grcr(nickname)    [::msgcat::mc "Nickname:"]
    pack_input $f 2 grcr(password)    [::msgcat::mc "Password:"]
    pack_input $f 3 grcr(name)        [string trim [::msgcat::mc "Name: "]]
    pack_input $f 4 grcr(description) [::msgcat::mc "Description:"]
    

    $w add -text [::msgcat::mc "Create"] -command "create_room $w"
    $w add -text [::msgcat::mc "Cancel"] -command "destroy $w"

    $w draw
}

proc conference::create_room {w} {
    global grcr

    destroy $w

    set server [server_from_jid $grcr(address)]

    jlib::send_iq get \
	[jlib::wrapper:createtag create \
	     -vars [list xmlns jabber:iq:conference]] \
	-to $server -command conference::create_room_recv_create

}

proc conference::create_room_recv_create {res child} {
    global grcr



}

