# $Id: login.tcl,v 1.41 2003/06/07 19:29:50 aleksey Exp $


if {![info exists loginconf(use_tls)]} {
    set loginconf(use_tls) 1
}
if {($loginconf(use_tls)) && ([catch { package require tls 1.4 }])} {
    debugmsg login "unable to load the TLS package, so no SSL support!

The TLS package is available at http://tls.sf.net/"

    set loginconf(use_tls) 0
}
if {!$loginconf(use_tls)} {
    set loginconf(usessl) 0
}

custom::defgroup Login \
    [::msgcat::mc "Login options."] \
    -group Tkabber

custom::defvar loginconf(user) "" \
    [::msgcat::mc "User name."] \
    -group Login -type string
custom::defvar loginconf(password) "" \
    [::msgcat::mc "Password."] \
    -group Login -type password
custom::defvar loginconf(resource) "tkabber" \
    [::msgcat::mc "Resource."] \
    -group Login -type string
custom::defvar loginconf(server) "localhost" \
    [::msgcat::mc "Server name."] \
    -group Login -type string
custom::defvar loginconf(port) "5222" \
    [::msgcat::mc "Server port."] \
    -group Login -type string
custom::defvar loginconf(priority) "8" \
    [::msgcat::mc "Priority."] \
    -group Login -type integer
custom::defvar loginconf(usessl) $loginconf(usessl) \
    [::msgcat::mc "Use SSL to connect to server."] \
    -group Login -type boolean
custom::defvar loginconf(sslport) "5223" \
    [::msgcat::mc "SSL port."] \
    -group Login -type string
custom::defvar loginconf(useproxy) 0 \
    [::msgcat::mc "Use HTTP proxy to connect."] \
    -group Login -type boolean
custom::defvar loginconf(httpproxy) "localhost" \
    [::msgcat::mc "HTTP proxy address."] \
    -group Login -type string
custom::defvar loginconf(httpproxyport) 3128 \
    [::msgcat::mc "HTTP proxy port."] \
    -group Login -type string
custom::defvar loginconf(httplogin) "" \
    [::msgcat::mc "HTTP proxy username."] \
    -group Login -type string
custom::defvar loginconf(httppassword) "" \
    [::msgcat::mc "HTTP proxy password."] \
    -group Login -type password
custom::defvar loginconf(usealtserver) 0 \
    [::msgcat::mc "Use explicitly-specified server address."] \
    -group Login -type boolean
custom::defvar loginconf(altserver) "" \
    [::msgcat::mc "Server name or IP-address."] \
    -group Login -type string

custom::defvar reasonlist {} [::msgcat::mc "List of logout reasons."] \
	-group Hidden


proc login {} {
    global loginconf
    global gr_nick gr_server gra_server
    global auth_result

    set gr_nick $loginconf(user)
    set gr_server conference.$loginconf(server)
    set gra_server conference.$loginconf(server)

    if {!$loginconf(useproxy)} {
	if {$loginconf(usealtserver)} {
	    set server $loginconf(altserver)
	} else {
	    set server $loginconf(server)
	}
	if {$loginconf(usessl)} {
	    set sock [tls::socket $server $loginconf(sslport)]
	} else {
	    set sock [socket $server $loginconf(port)]
	}
    } else {
	set sock [connect_httpproxy]
    }

    jlib::connect $sock $loginconf(server)

    jlib::wait_for_stream

    if {$loginconf(usedigest)} {
	set authtype digest
    } else {
	set authtype plain
    }
    jlib::send_auth $loginconf(user) $loginconf(password) $loginconf(resource) \
	recv_auth_result $authtype
    vwait auth_result

    if {$auth_result == "OK"} {
        connected
    }
}


set reconnect_retries 0

proc logout {} {
    global reconnect_retries

    jlib::disconnect
    roster::clean

    disconnected

    set reconnect_retries 0
}

proc client:disconnect {} {
    logout
}

proc client:reconnect {} {
    global reconnect_retries

    roster::clear .roster

    set_status "Disconnected"

    disconnected
    if {[incr reconnect_retries] <= 3} {
        after 1000
        login
    }
}

proc connected {} {
    global loginconf
    global userstatus textstatus
    global ssl_ind

    if {$loginconf(use_tls)} {
	if {$loginconf(usessl)} {
	    $ssl_ind configure -state normal
	    DynamicHelp::register $ssl_ind balloon [::msgcat::mc "SSL enabled"]
	} else {
	    $ssl_ind configure -state disabled
	    DynamicHelp::register $ssl_ind balloon [::msgcat::mc "SSL disabled"]
	}
    }

    wm title . "$loginconf(user)@$loginconf(server)/$loginconf(resource) - Tkabber"

    jlib::roster_get -command client:roster_cmd
    .presence.button configure -state normal
    set textstatus ""
    if {[cequal $userstatus unavailable]} {
        set userstatus available
    }
    set userstatus $userstatus
    hook::run connected_hook
}

proc disconnected {} {
    global loginconf
    global curuserstatus userstatusdesc
    global ssl_ind

    if {$loginconf(use_tls)} {
	$ssl_ind configure -state disabled
	DynamicHelp::register $ssl_ind balloon [::msgcat::mc "Disconnected"]
    }

    wm title . Tkabber

    set curuserstatus unavailable
    set userstatusdesc [::msgcat::mc "Not logged in"]
    .presence.button configure -state disabled
    hook::run change_our_presence_post_hook unavailable
    hook::run disconnected_hook
}

proc update_login_entries {l {i 0}} {
    global ltmp

    if {$i} {
	array set ltmp [array get ::loginconf$i]
    }
    foreach ent {username server port password resource priority \
	    altserver sslport httpproxy httpproxyport httplogin httppassword} {
	if {[winfo exists $l.$ent]} {
	    $l.$ent icursor end
	    if {[cequal [focus] $l.$ent]} {
		$l.$ent selection range 0 end
	    }
	}
    }
    foreach {check ents} {usessl sslport usealtserver altserver \
	    useproxy {httpproxy httpproxyport httplogin httppassword}} {
	if {[info exists ltmp($check)]} {
	    if {$ltmp($check)} {
		set state normal
	    } else {
		set state disabled
	    }
	    foreach ent $ents {
		if {[winfo exists $l.$ent]} {
		    $l.$ent configure -state $state
		    if {[cequal [focus] $l.$ent] && [cequal $state "disabled"]} {
			focus [Widget::focusPrev $l.$ent]
		    }
		}
	    }
	}
    }
}

proc show_login_dialog {} {
    global loginconf
    global ltmp
    #global tmpusername tmppassword tmpresource tmpserver tmpport tmppriority
    #global tmpusessl tmpsslport tmpuseproxy tmphttpproxy tmphttpproxyport

    if {[winfo exists .login]} {
	focus -force .login
	return
    }

    array set ltmp [array get loginconf]

    Dialog .login -title [::msgcat::mc "Login"] \
	-separator 1 -anchor e -default 0 -cancel 1

    set l [.login getframe]
    
    label $l.lusername -text [::msgcat::mc "Username:"]
    ecursor_entry [entry $l.username -textvariable ltmp(user)]
    label $l.lserver -text [::msgcat::mc "Server:"]
    ecursor_entry [entry $l.server -textvariable ltmp(server)]
    label $l.lport -text [::msgcat::mc "Port:"]
    ecursor_entry [entry $l.port -textvariable ltmp(port)]
    label $l.lpassword -text [::msgcat::mc "Password:"]
    ecursor_entry [entry $l.password -show * -textvariable ltmp(password)]
    checkbutton $l.usedigest -text [::msgcat::mc "Use hashed password"] \
	-variable ltmp(usedigest)
    label $l.lresource -text [::msgcat::mc "Resource:"]
    ecursor_entry [entry $l.resource -textvariable ltmp(resource)]
    label $l.lpriority -text [::msgcat::mc "Priority:"]
    ecursor_entry [entry $l.priority -textvariable ltmp(priority)]
    checkbutton $l.usealtserver -text [::msgcat::mc "Connect via alternate server"] \
	-variable ltmp(usealtserver) -command [list update_login_entries $l]
    label $l.laltserver -text [::msgcat::mc "Server:"]
    ecursor_entry [entry $l.altserver -textvariable ltmp(altserver)]

    grid $l.lusername -row 0 -column 0 -sticky e
    grid $l.username  -row 0 -column 1 -sticky ew
    grid $l.lserver   -row 1 -column 0 -sticky e
    grid $l.server    -row 1 -column 1 -sticky ew
    grid $l.usealtserver -row 4 -column 1 -sticky w -columnspan 3
    grid $l.laltserver -row 5 -column 0 -sticky e
    grid $l.altserver -row 5 -column 1 -sticky ew
    grid $l.lpassword -row 2 -column 0 -sticky e
    grid $l.password  -row 2 -column 1 -sticky ew
    grid $l.lresource -row 3 -column 0 -sticky e
    grid $l.resource  -row 3 -column 1 -sticky ew
    grid $l.usedigest -row 2 -column 3 -sticky e
    grid $l.lport     -row 1 -column 2 -sticky e
    grid $l.port      -row 1 -column 3 -sticky ew
    grid $l.lpriority -row 3 -column 2 -sticky e
    grid $l.priority  -row 3 -column 3 -sticky ew

    if {$loginconf(use_tls)} {
	checkbutton $l.usessl -text [::msgcat::mc "Use SSL"] \
	    -variable ltmp(usessl) -command [list update_login_entries $l]
	label $l.lsslport -text [::msgcat::mc "SSL Port:"]
	ecursor_entry [entry $l.sslport -textvariable ltmp(sslport)]

	grid $l.usessl   -row 6 -column 1 -sticky w
	grid $l.lsslport -row 6 -column 2 -sticky e
	grid $l.sslport  -row 6 -column 3 -sticky ew

    }

    checkbutton $l.useproxy -text [::msgcat::mc "Use Proxy"] \
	-variable ltmp(useproxy) -command [list update_login_entries $l]
    grid $l.useproxy -row 7 -column 1 -sticky w

    label $l.lhttpproxy -text [::msgcat::mc "Proxy Server:"]
    ecursor_entry [entry $l.httpproxy -textvariable ltmp(httpproxy)]
    label $l.lhttpproxyport -text [::msgcat::mc "Proxy Port:"]
    ecursor_entry [entry $l.httpproxyport -textvariable ltmp(httpproxyport)]

    grid $l.lhttpproxy     -row 8 -column 0 -sticky e
    grid $l.httpproxy      -row 8 -column 1 -sticky ew
    grid $l.lhttpproxyport -row 8 -column 2 -sticky e
    grid $l.httpproxyport  -row 8 -column 3 -sticky ew

    label $l.lhttplogin -text [::msgcat::mc "Proxy Login:"]
    ecursor_entry [entry $l.httplogin -textvariable ltmp(httplogin)]
    label $l.lhttppassword -text [::msgcat::mc "Proxy Password:"]
    ecursor_entry [entry $l.httppassword -show * -textvariable ltmp(httppassword)]

    grid $l.lhttplogin    -row 9 -column 0 -sticky e
    grid $l.httplogin     -row 9 -column 1 -sticky ew
    grid $l.lhttppassword -row 9 -column 2 -sticky e
    grid $l.httppassword  -row 9 -column 3 -sticky ew

    grid columnconfigure $l 1 -weight 1
    grid columnconfigure $l 3 -weight 1

    set n 1
    while {[info exists ::loginconf$n]} {incr n}
    incr n -1

    if {$n} {
	menubutton $l.profiles -text [::msgcat::mc Profiles] -relief raised \
	    -menu $l.profiles.menu
	set m [menu $l.profiles.menu -tearoff 0]
	for {set i 1} {$i <= $n} {incr i} {
	    if {[info exists ::loginconf${i}(profile)]} {
		set lab [set ::loginconf${i}(profile)]
	    } else {
		set lab "[::msgcat::mc Profile] $i"
	    }
	    if {$i <= 10} {
		set j [expr {$i % 10}]
		$m add command -label $lab -accelerator "Ctrl-$j" \
		    -command [list update_login_entries $l $i]
		bind .login <Control-Key-$j> [list update_login_entries $l $i]
	    } else {
		$m add command -label $lab \
		    -command [list update_login_entries $l $i]
	    }
	}
	#array set ltmp [array get ::loginconf1]

	grid $l.profiles -row 0 -column 3 -sticky e

    }

    .login add -text [::msgcat::mc "Log in"] -command {
	array set loginconf [array get ltmp]
	destroy .login
	logout
	update
	login
    }
    .login add -text [::msgcat::mc "Cancel"] -command {destroy .login}

    update_login_entries $l

    .login draw $l.password
}

package require base64

proc connect_httpproxy {} {
    global loginconf


    set sock [socket $loginconf(httpproxy) $loginconf(httpproxyport)]
    fconfigure $sock -buffering line

    if {$loginconf(usealtserver)} {
	set server $loginconf(altserver)
    } else {
	set server $loginconf(server)
    }
    if {$loginconf(usessl)} {
	puts $sock \
	    "CONNECT $server:${loginconf(sslport)} HTTP/1.0"
    } else {
	puts $sock "CONNECT $server:${loginconf(port)} HTTP/1.0"
    }
    
    if {$loginconf(httplogin) != ""} {
	set auth [base64::encode \
		      [encoding convertto "$loginconf(httplogin):$loginconf(httppassword)"]]
	puts $sock "Proxy-Authorization: Basic $auth"
    }
    puts $sock ""

    fileevent $sock readable {set proxy_readable ""}
    global proxy_readable
    vwait proxy_readable
    fileevent $sock readable {}

    set result [gets $sock]

    set code [lindex [split $result { }] 1]

    #debugmsg login $code
    if {$code >= 200 && $code < 300} {
	gets $sock
	if {$loginconf(usessl)} {
	    tls::import $sock
	}
	return $sock
    } else {
	error "proxy return: $result"
    }
}

proc recv_auth_result {res args} {
    global auth_result
    global loginconf

    if {$res == "OK"} {
	set auth_result OK
    } else {
	set auth_result ERR
	set res [MessageDlg .auth_err -aspect 50000 -icon error \
		-message [format \
		[::msgcat::mc "Authentication failed: %s\nCreate new account?"] \
		[error_to_string [lindex $args 0]]] -type user -buttons {yes no} -default 0 -cancel 1]
	if {!$res} {
	    jlib::send_iq set \
		[jlib::wrapper:createtag query \
		     -vars {xmlns jabber:iq:register} \
		     -subtags [list [jlib::wrapper:createtag username \
					 -chdata $loginconf(user)] \
				   [jlib::wrapper:createtag password \
					-chdata $loginconf(password)]]] \
		-command recv_register_result
	}
    }
}

proc recv_register_result {res args} {
    if {$res == "OK"} {
	jlib::disconnect
	login
    } else {
	MessageDlg .auth_err -aspect 50000 -icon error \
	    -message [format [::msgcat::mc "Registration failed: %s"] [error_to_string [lindex $args 0]]] \
	    -type user -buttons ok -default 0 -cancel 0
    }
}

proc change_password_dialog {} {
    global oldpassword newpassword password

    set oldpassword ""
    set newpassword ""
    set password ""

    if {[winfo exists .passwordchange]} {
	destroy .passwordchange
    }
    
    Dialog .passwordchange -title [::msgcat::mc "Change password"] \
	-separator 1 -anchor e -default 0 -cancel 1

    .passwordchange add -text [::msgcat::mc "OK"] -command {
	destroy .passwordchange
	send_change_password
    }
    .passwordchange add -text [::msgcat::mc "Cancel"] -command [list destroy .passwordchange]


    set p [.passwordchange getframe]
    
    label $p.loldpass -text [::msgcat::mc "Old password:"]
    ecursor_entry [entry $p.oldpass -show * -textvariable oldpassword]
    label $p.lnewpass -text [::msgcat::mc "New password:"]
    ecursor_entry [entry $p.newpass -show * -textvariable newpassword]
    label $p.lpassword -text [::msgcat::mc "Repeat new password:"]
    ecursor_entry [entry $p.password -show * -textvariable password]

    grid $p.loldpass  -row 0 -column 0 -sticky e
    grid $p.oldpass   -row 0 -column 1 -sticky ew
    grid $p.lnewpass  -row 1 -column 0 -sticky e
    grid $p.newpass   -row 1 -column 1 -sticky ew
    grid $p.lpassword -row 2 -column 0 -sticky e
    grid $p.password  -row 2 -column 1 -sticky ew

    focus $p.oldpass
    .passwordchange draw

}

proc send_change_password {} {
    global loginconf
    global oldpassword newpassword password

    if {$oldpassword != $loginconf(password)} {
	MessageDlg .auth_err -aspect 50000 -icon error \
	        -message [::msgcat::mc "Old password is incorrect"] \
		-type user -buttons ok -default 0 -cancel 0
	return
    }
    if {$newpassword != $password} {
	MessageDlg .auth_err -aspect 50000 -icon error \
	        -message [::msgcat::mc "New passwords do not match"] \
		-type user -buttons ok -default 0 -cancel 0
	return
    }

    jlib::send_iq set \
	    [jlib::wrapper:createtag query \
		    -vars {xmlns jabber:iq:register} \
		    -subtags [list [jlib::wrapper:createtag username \
					-chdata $loginconf(user)] \
				   [jlib::wrapper:createtag password \
					-chdata $password]]] \
	    -to $loginconf(server) \
	    -command recv_change_password_result
}

proc recv_change_password_result {res args} {
    global loginconf
    global newpassword

    if {$res == "OK"} {
	MessageDlg .shpasswd_result -aspect 50000 -icon info \
		-message [::msgcat::mc "Password is changed"] \
		-type user -buttons ok -default 0 -cancel 0
	for {set i 1} {[info exists ::loginconf$i]} {incr i} {
	    if {!([info exists ::loginconf${i}(user)] && \
		    [info exists ::loginconf${i}(server)] && \
		    [info exists ::loginconf${i}(password)])} {
		continue
	    }
	    upvar ::loginconf${i}(user) user
	    upvar ::loginconf${i}(server) server
	    upvar ::loginconf${i}(password) password
	    if {($user == $loginconf(user)) && \
		    ($server == $loginconf(server)) && \
		    ($password == $loginconf(password))} {
		set password $newpassword
	    }
	}
	set loginconf(password) $newpassword
    } else {
	MessageDlg .shpasswd_result -aspect 50000 -icon error \
	    -message [format [::msgcat::mc "Password change failed: %s"] [error_to_string [lindex $args 0]]] \
	    -type user -buttons ok -default 0 -cancel 0
    }
}

proc show_logout_dialog {} {
    global reason reasonlist

    set lw .logout

    if {![winfo exists $lw]} {
        Dialog $lw -title [::msgcat::mc "Logout with reason"] \
	    -separator 1 -anchor e -default 0 -cancel 1

        set lf [$lw getframe]
        grid columnconfigure $lf 1 -weight 1

	if {[llength $reasonlist]} {set reason [lindex $reasonlist 0]}

        label $lf.lreason   -text    [::msgcat::mc "Reason:"]
        ecursor_entry [ComboBox $lf.reason -textvariable reason \
		-values $reasonlist -width 35].e
        label $lf.lpriority -text    [::msgcat::mc "Priority:"]
        ecursor_entry [entry $lf.priority -textvariable loginconf(priority)]

        grid $lf.lreason   -row 0 -column 0 -sticky e
        grid $lf.reason    -row 0 -column 1 -sticky ew
        grid $lf.lpriority -row 1 -column 0 -sticky e
        grid $lf.priority  -row 1 -column 1 -sticky ew

        $lw add -text [::msgcat::mc "Log out"] -command logout_reason
        $lw add -text [::msgcat::mc "Cancel"] -command "$lw withdraw"
    } else {
        set lf [$lw getframe]
    }

    $lw draw $lf.reason
}

proc logout_reason {} {
    global userstatus textstatus reason reasonlist

    set reasonlist [update_combo_list $reasonlist $reason 10]
    set custom::saved(::::reasonlist) $reasonlist
    custom::store

    set lw .logout
    $lw withdraw

    set textstatus $reason
    set userstatus unavailable

    logout

    destroy $lw
}


