# $Id: rawxml.tcl,v 1.10 2003/02/22 19:10:51 alexey Exp $

option add *RawXML.inforeground       DarkRed     widgetDefault
option add *RawXML.outforeground      DarkBlue    widgetDefault
option add *RawXML.intagforeground    DarkRed     widgetDefault
option add *RawXML.inattrforeground   DarkRed     widgetDefault
option add *RawXML.invalueforeground  Purple4     widgetDefault
option add *RawXML.incdataforeground  SteelBlue   widgetDefault
option add *RawXML.outtagforeground   DarkMagenta widgetDefault
option add *RawXML.outattrforeground  DarkMagenta widgetDefault
option add *RawXML.outvalueforeground DarkGreen   widgetDefault
option add *RawXML.outcdataforeground DarkBlue    widgetDefault
option add *RawXML.inputheight        4           widgetDefault


namespace eval rawxml {
    custom::defgroup RawXML \
	[::msgcat::mc "Options for Raw XML Input module,\
which allows you to monitor\
incoming/outgoing traffic from connection to server and send\
custom XML stanzas."] -group Tkabber -tag "Raw XML Input"
	 
    #set options(pretty_print) 0
    custom::defvar options(pretty_print) 0 \
	[::msgcat::mc "Pretty print incoming and outgoing XML stanzas."] \
	-group RawXML -type boolean
    #set options(indent) 2
    custom::defvar options(indent) 2 \
	[::msgcat::mc "Indentation for pretty-printed XML subtags."] \
	-group RawXML -type integer

    set pass 0
    variable tabs
}


proc rawxml::handle_inout {text prefix tag} {
    variable options
    variable pass

    set w .rawxml
    if {![winfo exists $w]} return
    if {$options(pretty_print) && ($tag != "out" || $pass)} {
	set pass 0
	return
    }

    set dump $w.dump
    $dump configure -state normal

    $dump insert end "$prefix: " {} "$text" $tag

    if {![$dump compare "end -1 chars linestart" == "end -1 chars"]} {
	$dump insert end "\n"
    }

    $dump see end

    $dump configure -state disabled
}

proc rawxml::handle_inout_x {xml prefix tag} {
    variable options
    variable pass

    set w .rawxml
    if {![winfo exists $w]} return
    if {!$options(pretty_print)} return
    if {$tag == "out"} {set pass 1}

    set dump $w.dump
    $dump configure -state normal

    $dump insert end "$prefix:\n"
    #$dump insert end "$xml" in

    pretty_print $dump $xml "" $tag

    if {![$dump compare "end -1 chars linestart" == "end -1 chars"]} {
	$dump insert end "\n"
    }

    $dump see end

    $dump configure -state disabled
}

proc rawxml::pretty_print {t xmldata prefix tag} {
    global font
    variable options
    variable tabs

    jlib::wrapper:splitxml $xmldata tagname vars isempty chdata subtags

    $t insert end "$prefix<" {} $tagname ${tag}tag
    if {[llength $vars] != 0} {
	#set attrprefix ${prefix}[string repeat " " \
	#			     [expr {[clength $tagname] + 2}]]
	set arr_index "$prefix<$tagname "
	if {![info exists tabs($arr_index)]} {
	    set tabs($arr_index) [font measure $font $arr_index]
	}
	$t tag configure $arr_index -tabs [list $tabs($arr_index)]
	set vars1 [lassign $vars attr value]
	$t insert end \
	    " $attr" ${tag}attr \
	    "=" {} \
	    "'[jlib::wrapper:xmlcrypt $value]'" ${tag}value
	foreach {attr value} $vars1 {
	    $t insert end \
		"\n\t$attr" [list ${tag}attr $arr_index]\
		"=" {} \
		"'[jlib::wrapper:xmlcrypt $value]'" ${tag}value
	}
    }
    if {$chdata == "" && [llength $subtags] == 0} {
	$t insert end "/>\n"
	return
    } else {
	$t insert end ">"
    }

    if {$subtags == {}} {
	$t insert end [jlib::wrapper:xmlcrypt $chdata] ${tag}cdata
	$t insert end "</" {} $tagname ${tag}tag ">\n"
    } else {
	$t insert end "\n"
        foreach subdata $subtags {
	    pretty_print $t $subdata \
		$prefix[string repeat " " $options(indent)] $tag
	}
	$t insert end "$prefix</" {} $tagname ${tag}tag ">\n"
    }
}

proc ::LOG_INPUT {t} \
    "[namespace current]::rawxml::handle_inout \$t IN in"
proc ::LOG_OUTPUT {t} \
    "[namespace current]::rawxml::handle_inout \$t OUT out"
proc ::LOG_INPUT_XML {x} \
    "[namespace current]::rawxml::handle_inout_x \$x IN in"
proc ::LOG_OUTPUT_XML {x} \
    "[namespace current]::rawxml::handle_inout_x \$x OUT out"

proc rawxml::open_window {} {
    global font

    set w .rawxml
    if {[winfo exists $w]} {
	return
    }

    add_win $w -title [::msgcat::mc "Raw XML"] \
	-tabtitle [::msgcat::mc "Raw XML"] \
	-class RawXML \
	-raisecmd "focus [list $w.input]
		   tab_set_updated [list $w]"


    checkbutton $w.pp -text [::msgcat::mc "Pretty print XML"] \
	-variable [namespace current]::options(pretty_print)
    pack $w.pp -side top -anchor w

    PanedWin $w.pw -side right -pad 0 -width 8
    pack $w.pw -fill both -expand yes

    set uw [$w.pw add -weight 1 -minsize 100]
    set dw [$w.pw add -weight 0 -minsize 32]


    set isw [ScrolledWindow $w.isw -scrollbar vertical]
    pack $isw -side bottom -fill both -expand yes -in $dw
    set input [textUndoable $w.input -font $font -height [option get $w inputheight RawXML]]
    $isw setwidget $input
    [winfo parent $dw] configure -height [winfo reqheight $input]

    set sw [ScrolledWindow $w.sw -scrollbar vertical]
    pack $sw -side top -fill both -expand yes -in $uw
    set dump [text $w.dump -font $font]
    $sw setwidget $dump

    $dump configure -state disabled

    bind $input <Control-Key-Return> "
	[namespace current]::send_xml
	break"

    $dump tag configure in \
	-foreground [option get $w inforeground RawXML]
    $dump tag configure out \
	-foreground [option get $w outforeground RawXML]

    $dump tag configure intag \
	-foreground [option get $w intagforeground RawXML]
    $dump tag configure inattr \
	-foreground [option get $w inattrforeground RawXML]
    $dump tag configure invalue \
	-foreground [option get $w invalueforeground RawXML]
    $dump tag configure incdata \
	-foreground [option get $w incdataforeground RawXML]

    $dump tag configure outtag \
	-foreground [option get $w outtagforeground RawXML]
    $dump tag configure outattr \
	-foreground [option get $w outattrforeground RawXML]
    $dump tag configure outvalue \
	-foreground [option get $w outvalueforeground RawXML]
    $dump tag configure outcdata \
	-foreground [option get $w outcdataforeground RawXML]

    variable history
    bind $input <Control-Key-Up> \
	[list [namespace current]::history_move 1]
    bind $input <Control-Key-Down> \
	[list [namespace current]::history_move -1]

    set history(stack) [list {}]
    set history(pos) 0

    regsub -all %W [bind Text <Prior>] $dump prior_binding
    regsub -all %W [bind Text <Next>] $dump next_binding
    bind $input <Meta-Prior> $prior_binding
    bind $input <Meta-Next> $next_binding
    bind $input <Alt-Prior> $prior_binding
    bind $input <Alt-Next> $next_binding
}

proc rawxml::history_move {shift} {
    variable history

    set newpos [expr $history(pos) + $shift]

    if {!($newpos < 0 || $newpos >= [llength $history(stack)])} {
	set iw .rawxml.input
	set body [$iw get 1.0 "end -1 chars"]

	if {$history(pos) == 0} {
	    set history(stack) \
		[lreplace $history(stack) 0 0 $body]
	}

	set history(pos) $newpos
	set newbody [lindex $history(stack) $newpos]
	$iw delete 1.0 end
	$iw insert 0.0 $newbody
    }
}

proc rawxml::send_xml {} {
    variable history

    set input .rawxml.input
    set xml [$input get 0.0 end]

    lvarpush history(stack) $xml 1
    set history(pos) 0

    jlib::outmsg $xml
    $input delete 1.0 end
}


proc rawxml::setup_menu {} {
    catch { 
        set m [.mainframe getmenu admin]

        $m add command -label [::msgcat::mc "Open raw XML window"] \
	    -command [namespace current]::open_window
    }
}
hook::add finload_hook [namespace current]::rawxml::setup_menu

