# $Id: presence.tcl,v 1.38 2003/03/21 21:45:35 aleksey Exp $


proc client:presence {from type x args} {
    global presence

    debugmsg presence "PRESENCE: $from; $type; $x; $args"

    set from [tolower_node_and_domain $from]

    switch -- $type {
	error -
	unavailable {
	    foreach {attr val} $args {
		if {[cequal $attr -status]} {
		    set presence(status,$from) $val
		}
	    }
            if {[cequal $type unavailable]} {
	        presence_process_x $from $type $x
	    }

	    catch { unset presence(type,$from) }
	    #catch { unset presence(status,$from) }
	    catch { unset presence(priority,$from) }
	    catch { unset presence(meta,$from) }
	    catch { unset presence(icon,$from) }
	    catch { unset presence(show,$from) }
	    catch { unset presence(loc,$from) }
	    catch { unset presence(x,$from) }

	    set user [node_and_server_from_jid $from]
	    if {[info exists presence(user_jids,$user)]} {
		set idx [lsearch -exact $presence(user_jids,$user) $from]
		set presence(user_jids,$user) \
		    [lreplace $presence(user_jids,$user) $idx $idx]
	    }
	    cache_user_status $user

	    roster::on_change_jid_presence $from
	    debugmsg presence "[user_from_jid $from] unavailable"
	    chat::change_presence $from $type
	}
	subscribe {eval [list message::show_subscribe_dialog $from $x] $args}
	subscribed {}
	unsubscribe {}
	unsubscribed {
	    MessageDlg .unsubscribed -aspect 50000 -icon info \
		-title [format [::msgcat::mc "Unsubscribed from %s"] $from] \
		-message [format \
			      [::msgcat::mc "We unsubscribed from %s"] $from] \
		-type user -buttons {ok} -default 0 -cancel 0
	}
	probe {}
	default {
	    set presence(type,$from)     available
	    set presence(status,$from)   ""
	    set presence(priority,$from) 0
	    set presence(meta,$from)     ""
	    set presence(icon,$from)     ""
	    set presence(show,$from)     available
	    set presence(loc,$from)      ""
	    set presence(x,$from)        $x
	    
	    foreach {attr val} $args {
		switch -- $attr {
		    -status   {set presence(status,$from)   $val}
		    -priority {set presence(priority,$from) $val}
		    -meta     {set presence(meta,$from)     $val}
		    -icon     {set presence(icon,$from)     $val}
		    -show     {set presence(show,$from)     $val}
		    -loc      {set presence(loc,$from)      $val}
		}
	    }
	    
	    set presence(show,$from) [normalize_show $presence(show,$from)]
	    set show $presence(show,$from)

	    set user [node_and_server_from_jid $from]
	    lappend presence(user_jids,$user) $from
	    set presence(user_jids,$user) [lrmdups $presence(user_jids,$user)]

	    cache_user_status $user

	    presence_process_x $from available $x

	    roster::on_change_jid_presence $from
	    chat::change_presence $from $show
	}
    }
    
    hook::run client_presence_hook $from $type $x $args
}



proc get_jids_of_user {user} {
    global presence

    if {[info exists presence(user_jids,$user)]} {
	return $presence(user_jids,$user)
    } elseif {![cequal [resource_from_jid $user] ""]} {
	if {[info exists presence(type,$user)]} {
	    return [list $user]
	}
    }
    return {}
}

proc get_jid_of_user {user} {
    global presence

    set jids [get_jids_of_user $user]

    if {[llength $jids]} {
	set rjid [lindex $jids 0]
	set pr $presence(priority,$rjid)

	foreach jid $jids {
	    if {$presence(priority,$jid) > $pr} {
		set pr $presence(priority,$jid)
		set rjid $jid
	    }
	}

	return $rjid
    } else {
	return $user
    }
}

proc get_jid_status {jid} {
    global presence

    set j $jid
    if {[info exists presence(show,$j)]} {
	return $presence(show,$j)
    } else {
	return unavailable
    }
}

proc get_jid_presence_info {param jid} {
    global presence

    if {[info exists presence($param,$jid)]} {
	return $presence($param,$jid)
    } else {
	return ""
    }
}

proc get_user_status {user} {
    global presence

    if {[info exists presence(cachedstatus,$user)]} {
	return $presence(cachedstatus,$user)
    } elseif {[info exists presence(show,$user)]} {
	return $presence(show,$user)
    } else {
	return unavailable
    }
}

proc cache_user_status {user} {
    global presence

    set jid [get_jid_of_user $user]
    if {[info exists presence(show,$jid)]} {
	set presence(cachedstatus,$user) $presence(show,$jid)
    } else {
	set presence(cachedstatus,$user) unavailable
    }
}

proc get_user_status_desc {user} {
    global presence

    set jid [get_jid_of_user $user]
    if {[info exists presence(status,$jid)]} {
	return $presence(status,$jid)
    } else {
	return unavailable
    }
}

array set status_priority {
    stalker     0
    unavailable 1
    xa          2
    away        3
    dnd         4
    available   5
    chat        6
}

proc compare_status {s1 s2} {
    global status_priority
    set p1 $status_priority($s1)
    set p2 $status_priority($s2)
    if {$p1 > $p2} {
	return 1
    } elseif {$p1 == $p2} {
	return 0
    } else {
	return -1
    }
}

proc max_status {s1 s2} {
    global status_priority
    set p1 $status_priority($s1)
    set p2 $status_priority($s2)
    if {$p1 >= $p2} {
	return $s1
    } else {
	return $s2
    }
}

set curuserstatus [set userstatus available]
set curtextstatus [set textstatus ""]

set userstatusdesc [::msgcat::mc "Not logged in"]

set statusdesc(available) [::msgcat::mc "Online"]
set statusdesc(chat) [::msgcat::mc "Free to chat"]
set statusdesc(away) [::msgcat::mc "Away"]
set statusdesc(xa) [::msgcat::mc "Extended Away"]
set statusdesc(dnd) [::msgcat::mc "Do not disturb"]
set statusdesc(invisible) [::msgcat::mc "Invisible"]
set statusdesc(unavailable) [::msgcat::mc "Offline"]


trace variable userstatus w change_our_presence

proc change_our_presence {name1 name2 op} {
    global userstatus curuserstatus statusdesc userstatusdesc
    global textstatus curtextstatus
    global loginconf

    if {(![cequal $userstatus $curuserstatus]) \
	    || (![cequal $textstatus $curtextstatus])} {
	if {[lsearch -exact [array names statusdesc] $userstatus] < 0} {
	    error [cconcat [::msgcat::mc "invalid userstatus value "] $userstatus]
	}

	set userstatusdesc $statusdesc($userstatus)
	if {[cequal [set status $textstatus] ""]} {
	    set status $userstatusdesc
	}

	set p $loginconf(priority)
	set x [get_our_presence_xlist $status]

	switch -- $userstatus {
	    available {set command [list jlib::send_presence \
					-xlist $x -pri $p]}
            unavailable -
	    invisible {set command [list jlib::send_presence \
					-type $userstatus -xlist $x -pri $p]}
	    default {
		set command [list jlib::send_presence \
				 -show $userstatus -xlist $x -pri $p]
	    }
	}
        if {![cequal $status ""]} {
	    lappend command -stat $status
	}
	debugmsg presence $command
	eval $command
	if {$userstatus != "invisible"} {
	    foreach group [array names chat::opened] {
	        if {[cequal $chat::chats(type,$group) groupchat]} {
		    eval $command -to [list $group]
		}
	    }
	} else {
	    foreach group [array names chat::opened] {
	        if {[cequal $chat::chats(type,$group) groupchat]} {
		    jlib::send_presence \
		        -to $group/[get_our_groupchat_nick $group] -xlist $x
		}
	    }
	}

	set curuserstatus $userstatus
	set curtextstatus $textstatus
	hook::run change_our_presence_post_hook $userstatus
    }
}

proc send_custom_presence {jid status} {
    global loginconf
    global statusdesc

    set p $loginconf(priority)
    set x [get_our_presence_xlist $statusdesc($status)]

    switch -- $status {
	available {set command [list jlib::send_presence -to $jid \
				    -xlist $x -pri $p]}
	unavailable {set command [list jlib::send_presence -to $jid \
				      -type $status -xlist $x -pri $p]}
	default {
	    set command [list jlib::send_presence -to $jid \
			     -show $status -xlist $x -pri $p]
	}
    }
    lappend command -stat $statusdesc($status)

    debugmsg presence $command
    eval $command
}

proc get_our_presence_xlist {status} {
    set xs {}

    foreach x [list [avatar::get_presence_x] \
		    [make_signature $status]] {
	if {$x != {}} {
	    lappend xs $x
	}
    }

    return $xs
}

proc normalize_show {show} {
    set res $show
    
    switch -- $show {
	away        {}
	chat        {}
    	dnd         {}
	xa          {}
	unavailable {}
	stalker     {}
	default     {set res available}
    }
    return $res
}


proc presence_process_x {from type xs} {
    catch { unset roster::roster(signed,$from) }
    foreach x $xs {
	jlib::wrapper:splitxml $x tag vars isempty chdata children
	switch -- [jlib::wrapper:getattr $vars xmlns] {
	    jabber:x:avatar {
		avatar::set_hash $from $children
	    }

	    jabber:x:signed {
		check_signature $from $chdata
	    }

	    "http://jabber.org/protocol/muc#user" {
		muc::process_gc_user $from $type $children
	    }
	}
    }
}

proc check_signature {from signature} {
    global presence

# in case the sender didn't check the exit code from gpg...
    if {([cequal $signature ""]) \
	    || ([cequal [info commands ::ssj::signed:input] ""])} {
	return
    }

    set roster::roster(signed,$from) \
	[ssj::signed:input $from $signature $presence(status,$from) \
		"Presence information"]
}

proc make_signature {status} {
    if {(![cequal [info commands ::ssj::signed:output] ""]) \
	    && (![catch { ssj::signed:output $status } chdata]) \
	    && (![cequal $chdata ""])} {
	return [jlib::wrapper:createtag x -vars {xmlns jabber:x:signed} \
			-chdata $chdata]
    }
}


proc clear_presence_info {} {
    global presence

    array unset presence
}
hook::add disconnected_hook clear_presence_info
