# $Id: privacy.tcl,v 1.5 2003/01/09 19:51:29 alexey Exp $

namespace eval privacy {}

set ::NS(privacy) jabber:iq:privacy


proc privacy::request_lists {} {
    jlib::send_iq get \
	[jlib::wrapper:createtag query \
	     -vars [list xmlns $::NS(privacy)]] \
	-command [list [namespace current]::open_dialog]
}


proc privacy::open_dialog {res child} {
    variable rf

    if {![cequal $res OK]} {
	MessageDlg .privacy_err -aspect 50000 -icon error \
	    -message [format [::msgcat::mc "Requesting privacy rules: %s"] \
			  [error_to_string $child]] \
	    -type user -buttons ok -default 0 -cancel 0
	return
    }

    set w .privacy

    if {[winfo exists $w]} {
	destroy $w
    }


    Dialog $w -title [::msgcat::mc "Zebra lists"] \
	-separator 1 -anchor e \
	-default 0 -cancel 1

    set f [$w getframe]

    set sw [ScrolledWindow $w.sw]
    set sf [ScrollableFrame $w.fields -constrainedwidth yes]
    pack $sw -side top -expand yes -fill both
    set lf [$sf getframe]
    $sw setwidget $sf

    #set lf1 [frame $lf.lists]
    #pack $lf1 -side top -fill both -expand yes

    jlib::wrapper:splitxml $child tag vars isempty chdata childrens
    fill_lists $lf $childrens

    set tools [frame $f.tools]
    pack $tools -side top -fill x
    
    set addlist [button $tools.addlist \
		     -text [::msgcat::mc "Add list"] \
		     -command [list [namespace current]::add_list $lf "" {}]]
    pack $addlist -side left

    set active [radiobutton $tools.active \
		    -text [::msgcat::mc "No active"] \
		    -variable [namespace current]::data(active) \
		    -value ""]
    pack $active -side left

    $w add -text [::msgcat::mc "Send"] \
	-command [list [namespace current]::send_lists $w]
    $w add -text [::msgcat::mc "Cancel"] -command [list destroy $w]

    after 1 [list $sw setwidget $sf]
    $w draw
}

proc privacy::fill_lists {f items} {
    variable data

    set data(active) ""
    set data(nlists) 0
    set i 0
    foreach item $items {
	jlib::wrapper:splitxml $item tag vars isempty chdata childrens
	switch -- $tag {
	    active {
		set data(active) [jlib::wrapper:getattr $vars name]
	    }
	    list {
		set name [jlib::wrapper:getattr $vars name]

		add_list $f $name $childrens
	    }
	}
    }
}

proc privacy::fill_list {f ln items} {
    variable data

    set data(nitems,$ln) 0

    label $f.ljid   -text [::msgcat::mc "JID:"]
    label $f.ltype  -text [::msgcat::mc "Type:"]
    label $f.lsubsc -text [::msgcat::mc "Subscription:"]
    grid $f.ljid   -row 0 -column 0 -sticky w
    grid $f.ltype  -row 0 -column 1 -sticky w
    grid $f.lsubsc -row 0 -column 2 -sticky w
    grid columnconfig $f 0 -weight 1

    foreach item $items {
	jlib::wrapper:splitxml $item tag vars isempty chdata childrens

	set jid [jlib::wrapper:getattr $vars jid]
	set subsc [jlib::wrapper:getattr $vars subscription]
	set type [jlib::wrapper:getattr $vars type]

	if {$subsc != "to" && $subsc != "from"} {
	    set subsc "none"
	}

	if {$type != "deny"} {
	    set type "allow"
	}

	add_item $f $ln $jid $subsc $type
    }
}


proc privacy::add_item {f ln jid subsc type} {
    variable data

    set i $data(nitems,$ln)

    set data(jid,$i,$ln)   $jid
    set data(subsc,$i,$ln) $subsc
    set data(type,$i,$ln)  $type

    entry $f.jid$i \
	-textvariable [namespace current]::data(jid,$i,$ln)
    tk_optionMenu $f.subsc$i \
	[namespace current]::data(subsc,$i,$ln) \
	none from to
    tk_optionMenu $f.type$i \
	[namespace current]::data(type,$i,$ln) \
	allow deny
    button $f.remove$i -text [::msgcat::mc "Remove"] \
	-command [list [namespace current]::remove_item $f $ln $i]

    set row [expr {$i + 1}]
    grid $f.jid$i    -row $row -column 0 -sticky ew
    grid $f.type$i   -row $row -column 1 -sticky ew
    grid $f.subsc$i  -row $row -column 2 -sticky ew
    grid $f.remove$i -row $row -column 3 -sticky ew

    incr data(nitems,$ln)
}

proc privacy::remove_item {f ln i} {
    variable data

    destroy $f.jid$i
    destroy $f.type$i
    destroy $f.subsc$i
    destroy $f.remove$i

    set data(jid,$i,$ln) ""
    set data(subsc,$i,$ln) none
    set data(type,$i,$ln) remove
}

proc privacy::remove_list {tf ln} {
    variable data

    destroy $tf
    set data(nitems,$ln) 0
    set data(newname,$ln) ""
}

proc privacy::add_list {f name childrens} {
    variable data

    set i $data(nlists)

    if {$name == ""} {
	set name "list$i"
    }

    set data(name,$i) $name
    set data(newname,$i) $name

    set tf [TitleFrame $f.list$i \
		-text [format [::msgcat::mc "Zebra list: %s"] $name] \
		-borderwidth 2 \
		-relief groove]
    pack $tf -side top -fill x
    set lf [$tf getframe]
    set itemsf [frame $lf.items]
    pack $itemsf -side bottom -fill both -expand yes
    fill_list $itemsf $i $childrens

    set active [radiobutton $lf.active \
		    -text [::msgcat::mc "Active"] \
		    -variable [namespace current]::data(active) \
		    -value $name]
    pack $active -side top -anchor w
    
    set remove [button $lf.remove \
		    -text [::msgcat::mc "Remove list"] \
		    -command \
		    [list [namespace current]::remove_list \
			 $tf $i]]
    pack $remove -side top -anchor w
    
    set additem [button $lf.aditem \
		     -text [::msgcat::mc "Add item"] \
		     -command \
		     [list [namespace current]::add_item \
			  $itemsf $i "" "none" "allow"]]
    pack $additem -side top -anchor w
    
    label $lf.lname -text [string trimright [::msgcat::mc "Name: "]]
    entry $lf.name \
	-textvariable [namespace current]::data(newname,$i)
    pack $lf.lname -side left -anchor w
    pack $lf.name -side left -fill x -expand yes
    
    incr data(nlists)
}

proc privacy::send_lists {w} {
    variable data

    #set items {}

    for {set i 0} {$i < $data(nlists)} {incr i} {
	set name $data(newname,$i)
	if {$name != $data(name,$i) || $name == ""} {
	    set items [list [jlib::wrapper:createtag list \
				 -vars [list name $data(name,$i)]]]
	    jlib::send_iq set \
		[jlib::wrapper:createtag query \
		     -vars [list xmlns $::NS(privacy)] \
		     -subtags $items]
	    if {$name == ""} continue
	}

	set litems [list_items $i]

	if {$litems != {}} {
	    set items [list [jlib::wrapper:createtag list \
				 -vars [list name $data(name,$i)] \
				 -subtags $litems]]
	    jlib::send_iq set \
		[jlib::wrapper:createtag query \
		     -vars [list xmlns $::NS(privacy)] \
		     -subtags $items]
	}
    }

    if {$data(active) != ""} {
	set items [list [jlib::wrapper:createtag active \
			     -vars [list name $data(active)]]]
    } else {
	set items [list [jlib::wrapper:createtag active]]
    }
    jlib::send_iq set \
	[jlib::wrapper:createtag query \
	     -vars [list xmlns $::NS(privacy)] \
	     -subtags $items]
    destroy $w
}

proc privacy::list_items {ln} {
    variable data

    set items {}
    for {set i 0} {$i < $data(nitems,$ln)} {incr i} {
	if {$data(type,$i,$ln) == "remove"} continue

	set vars [list type $data(type,$i,$ln)]
	if {$data(jid,$i,$ln) != ""} {
	    lappend vars jid $data(jid,$i,$ln)
	}
	if {$data(subsc,$i,$ln) != "none"} {
	    lappend vars subscription $data(subsc,$i,$ln)
	}
	lappend items [jlib::wrapper:createtag item \
			   -vars $vars]
    }
    return $items
}

