# $Id: search.tcl,v 1.25 2003/06/04 07:22:46 aleksey Exp $

namespace eval search {
    set winid 0
    set show_all 0
}

option add *Search.itemBackground White	widgetDefault

proc search::open {jid} {
    variable winid
    variable config

    set sw .search$winid

    toplevel $sw -cursor watch
    wm title $sw "[::msgcat::mc {Search in}] $jid"

    ButtonBox $sw.bbox -spacing 0 -padx 10 -default 0
    $sw.bbox add -text [::msgcat::mc "OK"] -command [list search::search $sw $jid] \
        -state disabled
    $sw.bbox add -text [::msgcat::mc "Cancel"] -command [list destroy $sw]
    pack $sw.bbox -padx 2m -pady 2m -anchor e -side bottom

    bind $sw <Return> "ButtonBox::invoke $sw.bbox default"
    bind $sw <Escape> "ButtonBox::invoke $sw.bbox 1"

    Separator::create $sw.sep -orient horizontal
    pack $sw.sep -side bottom -fill x -pady 1m
    
    frame $sw.fields -class Search

    set config(itemBackground) [option get $sw.fields itemBackground Search]

    pack $sw.fields -expand yes -fill both -padx 2m -pady 2m

    wm withdraw $sw

    jlib::send_iq get \
	[jlib::wrapper:createtag query \
	     -vars [list xmlns jabber:iq:search \
			xml:lang [get_lang]]] \
	-to $jid -command [list search::recv_fields $sw $jid]
    
    incr winid
}


proc search::recv_fields {sw jid res child} {
    debugmsg search "$res $child"

    if {![cequal $res OK]} {
	return
    }

    jlib::wrapper:splitxml $child tag vars isempty chdata children

    if {[cequal [jlib::wrapper:getattr $vars xmlns] jabber:iq:search]} {
	data::fill_fields $sw.fields $children
    }

    $sw configure -cursor {}
    $sw.bbox itemconfigure 0 -state normal
    wm deiconify $sw

    foreach child [winfo children $sw.fields] {
        if {[cequal [winfo class $child] Entry]} {
	    focus $child
	    break
	}
    }
}


proc search::search {sw jid} {
    variable data

    $sw configure -cursor watch
    $sw.bbox itemconfigure 0 -state disabled

    set f $sw.fields
    set restags [data::get_tags $sw.fields]

    jlib::send_iq set [jlib::wrapper:createtag query \
			   -vars [list xmlns jabber:iq:search \
				      xml:lang [get_lang]] \
			   -subtags $restags] \
	-to $jid -command [list search::recv_items $sw $jid]
}


proc search::recv_items {sw jid res child} {
    variable config
    global font
    variable lastsort

    debugmsg search "$res $child"

    if {![winfo exists $sw]} {
	return
    }

    if {![cequal $res OK]} {
	$sw configure -cursor {}
	$sw.bbox itemconfigure 0 -state normal \
	    -command [list search::search_again $sw $jid errormsg] \
	    -text [::msgcat::mc "Try again"]
	$sw.bbox itemconfigure 1 -state normal \
	    -text [::msgcat::mc "Close"]

	if {[winfo exists $sw.errormsg]} {
	    destroy $sw.errormsg
	}

	message $sw.errormsg -aspect 50000 -font $font \
	    -text [format [::msgcat::mc "An error is occured when searching in %s\n\n%s"] $jid [error_to_string $child]]

	pack $sw.errormsg -expand yes -fill both -after $sw.fields -anchor nw -padx 4m -pady 4m
	pack forget $sw.fields

	return
    }

    $sw configure -cursor {}
    $sw.bbox itemconfigure 0 -state normal \
	-command [list search::search_again $sw $jid items] \
	-text [::msgcat::mc "Search again"]
    $sw.bbox itemconfigure 1 -state normal \
	-text [::msgcat::mc "Close"]

    if {[winfo exists $sw.items]} {
	destroy $sw.items
    }

    set sww [ScrolledWindow $sw.items]

    ::mclistbox::mclistbox $sww.listbox \
	-resizeonecolumn 1 \
	-font $font \
	-labelfont $font \
	-labelanchor w \
	-width 90 \
	-height 16

    $sww setwidget $sww.listbox

    pack $sww.listbox -expand yes -fill both
    pack $sww -expand yes -fill both -after $sw.fields -anchor nw -padx 2m -pady 2m
    pack forget $sw.fields

    set lastsort($sww.listbox) ""
    bind $sww.listbox <Destroy> +[list [namespace current]::delete_lastsort $sww.listbox]
    
    bind $sww.listbox <3> \
	"[namespace current]::select_and_popup_menu $sww.listbox \[$sww.listbox nearest \[::mclistbox::convert %W -y %y\]\]"

    bind $sww <4> "$sww.listbox yview scroll -1 units"
    bind $sww <5> "$sww.listbox yview scroll 1 units"
    bind $sww <6> "$sww.listbox xview scroll -1 units"
    bind $sww <7> "$sww.listbox xview scroll 1 units"

    set reported_fields [data::get_reported_fields $sw.fields]

    jlib::wrapper:splitxml $child tag vars isempty chdata children

    if {[cequal [jlib::wrapper:getattr $vars xmlns] jabber:iq:search]} {
	fill_mclistbox $sw $jid $sww.listbox $reported_fields $children
    }
}

proc search::delete_lastsort {id} {
    variable lastsort

    if {[info exists lastsort($id)]} {
	unset lastsort($id)
    }
}

proc search::fill_mclistbox_x {sw jid w reported_fields items} {
    variable show_all

    set width(0) 3
    set name(0) N
    $w column add N -label " [::msgcat::mc #] "

    set row 0
    set col 1

    foreach item $items {
	jlib::wrapper:splitxml $item tag vars isempty chdata children

	switch -- $tag {
	    title {
		if {$chdata != ""} {
		    wm title $sw $chdata
		}
	    }
	    reported {
		set reported_fields {}
		foreach field $children {
		    jlib::wrapper:splitxml $field tag vars isempty chdata children1

		    set lname [jlib::wrapper:getattr $vars "var"]
		    set label_name($lname) [jlib::wrapper:getattr $vars "label"]
		    lappend reported_fields $lname
		}
	    }
	    item {
		foreach field $children {
		    jlib::wrapper:splitxml $field tag vars isempty chdata children1

		    if {$tag == "field"} {
			set var [jlib::wrapper:getattr $vars "var"]

			foreach value $children1 {
			    
			    jlib::wrapper:splitxml $value tag vars isempty chdata children1

			    if {($tag == "value") && ($chdata != "")} {
				if {$show_all || ([lsearch -exact $reported_fields $var] != -1)} {
				    if {![info exists fieldcol($var)]} {
					set fieldcol($var) $col
					if {[info exists label_name($var)]} {
					    set l $label_name($var)
					} else {
					    set l $var
					}
					set width($col) [string length " $l "]
					set name($col) $var
					$w column add $var -label " $l "
					$w label bind $var <ButtonPress-1> "[namespace current]::sort %W $var"
					set lasttag $var

					incr col
				    }
				    set data($fieldcol($var),$row) $chdata

				    debugmsg search "$var $chdata"
				}
			    }
			}
		    }
		}
		set data(0,$row) [expr {$row + 1}]
		incr row
	    }
	    default {}
	}	
    }

    finalize_mclistbox $w $row $col name data width
}

proc search::finalize_mclistbox {w row col n d wi} {
    upvar $n name
    upvar $d data
    upvar $wi width

    $w column add lastcol -label "" -width 0
    $w configure -fillcolumn lastcol
    
    for {set j 0} {$j < $row} {incr j} {
	set datalist {}
	for {set i 0} {$i < $col} {incr i} {
	    if {[info exists data($i,$j)]} {
		set wd [string length " $data($i,$j) "]
		if {$wd > $width($i)} {
		    set width($i) $wd
		}
		lappend datalist " $data($i,$j) "
	    } else {
		lappend datalist ""
	    }
	}
	lappend datalist ""
	$w insert end $datalist
    }
    for {set i 0} {$i < $col} {incr i} {
	$w column configure $name($i) -width $width($i)
    }
}

proc search::fill_mclistbox {sw jid w reported_fields items} {
    variable show_all

    foreach item $items {
	jlib::wrapper:splitxml $item tag vars isempty chdata children
	if {$tag == "x" &&
	    [jlib::wrapper:getattr $vars xmlns] == "jabber:x:data"} {
	    fill_mclistbox_x $sw $jid $w $reported_fields $children
	    return
	}
    }

    set width(0) 3
    set name(0) N
    $w column add N -label " [::msgcat::mc #] "

    set fieldcol(jid) 1
    set width(1) 5
    set name(1) jid
    $w column add jid -label " jid "
    $w label bind jid <ButtonPress-1> "[namespace current]::sort %W jid"
 
    set row 0
    set col 2

    foreach item $items {
	jlib::wrapper:splitxml $item tag vars isempty chdata children

	switch -- $tag {
	    item {
		set itemjid [jlib::wrapper:getattr $vars jid]
		set data(1,$row) $itemjid

		foreach field $children {
		    jlib::wrapper:splitxml $field tag vars isempty \
			chdata children1

		    if {$chdata != ""} {
			if {$show_all || ([lsearch -exact $reported_fields $tag] != -1)} {
			    if {![info exists fieldcol($tag)]} {
				set fieldcol($tag) $col
				set width($col) [string length " $tag "]
				set name($col) $tag
				$w column add $tag -label " $tag "
				$w label bind $tag <ButtonPress-1> "[namespace current]::sort %W $tag"
				set lasttag $tag

				incr col
			    }
			    set data($fieldcol($tag),$row) $chdata

			    debugmsg search "$tag $chdata"
			}
		    }
		}
		set data(0,$row) [expr {$row + 1}]
		incr row
	    }
	    default {}
	}	
    }

    finalize_mclistbox $w $row $col name data width
}

proc search::sort {w tag} {
    variable lastsort

    #debugmsg search $w

    set data [$w get 0 end]
    set index [lsearch -exact [$w column names] $tag]
    if {$lastsort($w) != $tag} {
	set result [lsort -dictionary -index $index $data]
	set lastsort($w) $tag
    } else {
	set result [lsort -decreasing -dictionary -index $index $data]
	set lastsort($w) ""
    }
    set result1 {}
    set i 0
    foreach row $result {
	lappend result1 [lreplace $row 0 0 " [incr i] "]

    }
    $w delete 0 end
    eval $w insert end $result1
}

proc search::search_again {sw jid delwidget} {
    pack $sw.fields -expand yes -fill both -after $sw.$delwidget -anchor nw -padx 2m -pady 4m
    pack forget $sw.$delwidget

    $sw configure -cursor {}
    $sw.bbox itemconfigure 0 -text [::msgcat::mc "OK"] -command [list search::search $sw $jid] \
	-state normal
    $sw.bbox itemconfigure 1 -state normal \
	-text [::msgcat::mc "Cancel"]
}

proc search::select_and_popup_menu {w index} {
    $w selection clear 0 end
    $w selection set $index
    set jid [string trim [lindex [$w get $index] 1]]

    if {[winfo exists [set m .searchpopupmenu]]} {
	destroy $m
    }
    menu $m -tearoff 0
    $m add command -label [::msgcat::mc "Start chat"] \
	-command [list chat::open_to_user $jid]
    $m add command -label [::msgcat::mc "Send message..."] \
	-command [list message::send_dialog $jid]
    $m add command -label [::msgcat::mc "Invite to conference..."] \
	-command [list chat::invite_dialog $jid]
#    hook::run roster_create_groupchat_user_menu_hook $m
    $m add separator
    $m add command -label [::msgcat::mc "Send users..."] \
	-command [list roster::send_users_dialog $jid]
    $m add command -label [::msgcat::mc "Send file..."] \
	-command [list ft::send_file_dialog $jid]
    $m add command -label [::msgcat::mc "Send file via Jidlink..."] \
	-command [list ftjl::send_file_dialog $jid]
    $m add separator
    $m add command -label [::msgcat::mc "Add user..."] \
	-command [list message::send_subscribe_dialog $jid]
    $m add command -label [::msgcat::mc "Show info..."] \
	-command [list userinfo::open $jid]
    $m add command -label [::msgcat::mc "Show history..."] -command {} -state disabled

    tk_popup $m [winfo pointerx .] [winfo pointery .]
}

hook::add postload_hook \
    {browser::register_ns_handler jabber:iq:search search::open \
    -desc "[::msgcat::mc {Search}]"}
hook::add postload_hook \
    {disco::browser::register_feature_handler jabber:iq:search search::open \
    -desc "[::msgcat::mc {Search}]"}
