# $Id: utils.tcl,v 1.15 2003/06/04 07:22:46 aleksey Exp $

proc user_from_jid {jid} {
    set user $jid
    regexp {(.*@.*)/.*} $jid temp user

    return $user
}

proc node_and_server_from_jid {jid} {
    set nas $jid
    regexp {([^/]*)/.*} $jid temp nas

    return $nas
}

proc server_from_jid {jid} {
    set serv $jid
    regexp {([^/]*)/.*} $jid temp serv
    regexp {[^@]*@(.*)} $jid temp serv

    return $serv
}

proc resource_from_jid {jid} {
    set resource ""
    regexp {[^/]*/(.*)} $jid temp resource

    return $resource
}

proc node_from_jid {jid} {
    set node ""
    regexp {([^@]*)@.*} $jid temp node
    #debugmsg utils "Node($jid) = $node"

    return $node
}


proc tolower_node_and_domain {jid} {
    
    set nas [string tolower [node_and_server_from_jid $jid]]
    set resource [resource_from_jid $jid]

    if {![cequal $resource ""]} {
	return $nas/$resource
    } else {
	return $nas
    }
    
}



proc win_id {prefix key} {
    global wins

    if {![info exists wins(seq,$prefix)]} {
	set wins(seq,$prefix) 0
    }

    if {![info exists wins(key,$prefix,$key)]} {
	set idx $wins(seq,$prefix)
	set wins(key,$prefix,$key) ".${prefix}_$idx"
	incr wins(seq,$prefix)
    }
    return $wins(key,$prefix,$key)
}


proc jid_to_tag {jid} {
    variable jidtag
    variable tagjid
    
    if {[info exists jidtag($jid)]} {
	return $jidtag($jid)
    } else {
	set tag [join [lmatch -regexp [split $jid {}] \
			   {^[[:alpha:]]$}] {}][random 1000000000]

	while {[info exists tagjid($tag)]} {
	    set tag [join [lmatch -regexp [split $jid {}] \
			       {^[[:alpha:]]$}] {}][random 1000000000]
	}

	set jidtag($jid) $tag
	set tagjid($tag) $jid

	return $tag
    }
}

proc tag_to_jid {tag} {
    variable tagjid

    if {[info exists tagjid($tag)]} {
	return $tagjid($tag)
    } else {
	error "Unknown tag $tag"
    }
}

proc double% {str} {
    regsub -all % $str %% res
    return $res
}

proc get_lang {} {
    set prefs [::msgcat::mcpreferences]
    lindex $prefs [expr {[llength $prefs] - 1}]
}

set ::NS(stanzas) "urn:ietf:params:xml:ns:xmpp-stanzas"

array set ::error_class [list \
			     access    "Access Error" \
			     address   "Address Error" \
			     app       "Application Error" \
			     format    "Format Error" \
			     recipient "Recipient Error" \
			     server    "Server Error"]

foreach {class descelem description} \
    [list \
	 format    bad-request             "Bad Request" \
	 access    conflict                "Conflict" \
	 recipient feature-not-implemented "Feature Not Implemented" \
	 access    forbidden               "Forbidden" \
	 server    internal-server-error   "Internal Server Error" \
	 address   jid-malformed           "JID Malformed" \
	 address   item-not-found          "Item Not Found" \
	 access    not-allowed             "Not Allowed" \
	 recipient recipient-unavailable   "Recipient Unavailable" \
	 access    registration-required   "Registration Required" \
	 address   remote-server-not-found "Remote Server Not Found" \
	 server    remote-server-timeout   "Remote Server Timeout" \
	 server    service-unavailable     "Service Unavailable"] \
    {
	set ::error_description($class,$::NS(stanzas),$descelem) $description
    }

array set ::error_class_descelem [list \
	400 {format    bad-request} \
	403 {access    forbidden} \
	404 {address   item-not-found} \
	405 {access    not-allowed} \
	407 {access    registration-required} \
	409 {access    conflict} \
	500 {server    internal-server-error} \
	501 {recipient feature-not-implemented} \
	503 {server    service-unavailable} \
	504 {server    remote-server-timeout}]

proc error_to_string {errmsg} {

    return [lindex [error_to_list $errmsg] 2]
}

proc error_to_list {errmsg} {
    global error_class error_description error_class_descelem

    lassign $errmsg code desc
    if {[string is integer $code]} {
	if {[info exists error_class_descelem($code)]} {
	    lassign $error_class_descelem($code) class descelem
	} else {
	    lassign {none none} class descelem
	}
	return [list $class $descelem "$code ([::msgcat::mc $desc])"]
    } else {
	set class $code
	set errelem $desc
	jlib::wrapper:splitxml $errelem tag vars isempty chdata childrens
	lassign $childrens stanza_condition
	set description ""
	if {$stanza_condition != {}} {
	    jlib::wrapper:splitxml $stanza_condition \
		tag1 vars1 isempty1 chdata1 childrens1
	    set xmlns [jlib::wrapper:getattr $vars1 xmlns]
	    lassign $childrens1 condelem
	    jlib::wrapper:splitxml $condelem \
		tag2 vars2 isempty2 chdata2 childrens2
	    set condition $tag2

	    if {[info exists error_description($class,$xmlns,$condition)]} {
		set description $error_description($class,$xmlns,$condition)
	    }
	}
	if {[info exists error_class($class)]} {
	    set classdesc $error_class($class)
	}
	set res ""
	if {$classdesc != ""} {
	    set res "[::msgcat::mc $classdesc]"
	}
	if {$description != ""} {
	    set res "${res} ([::msgcat::mc $description])"
	}
	return [list $class $condition $res]
    }
}


proc get_group_nick {jid fallback} {
    global defaultnick

    set nick $fallback
    set tmp_pattern *
    foreach pattern [array names defaultnick] {
	if {[string equal $pattern $jid]} {
	    return $defaultnick($pattern)
	} elseif {([string match $pattern $jid]) && ([string match $tmp_pattern $pattern])} {
	    set nick $defaultnick($pattern)
	    set tmp_pattern $pattern
	}
    }
    return $nick
}

proc format_time {t} {
	if {[cequal $t ""]} {
	    return
	}

	set sec [expr {$t % 60}]
	set secs [expr {($sec==1)?"[msgcat::mc second]":"[msgcat::mc seconds]"}]
	set t [expr {$t / 60}]
	set min [expr {$t % 60}]
	set mins [expr {($min==1)?"[msgcat::mc minute]":"[msgcat::mc minutes]"}]
	set t [expr {$t / 60}]
	set hour [expr {$t % 24}]
	set hours [expr {($hour==1)?"[msgcat::mc hour]":"[msgcat::mc hours]"}]
	set day [expr {$t / 24}]
	set days [expr {($day==1)?"[msgcat::mc day]":"[msgcat::mc days]"}]

	set flag 0
	set message ""
	if {$day != 0} {
		set flag 1
		set message "$day $days"
	}
	if {$flag || ($hour != 0)} {
		set flag 1
		set message [concat $message "$hour $hours"]
	}
	if {$flag || ($min != 0)} {
		set message [concat $message "$min $mins"]
	}

	return [concat $message "$sec $secs"]
}

proc NonmodalMessageDlg {path args} {

    set icon "none"
    set title ""
    set message ""
    set opts {}
    set mopts {}
    foreach {option value} $args {
	switch -- $option {
	    -icon {
		set icon $value
		if { $value != "none" } {
		    set image [Bitmap::get $value]
		} else {
		    set image ""
		}
	    }
	    -title {
		set title $value
	    }
	    -aspect {
		lappend mopts $option $value
	    }
	    -message {
		lappend mopts -text $value
	    }
	    default {
		lappend opts $option $value
	    }
	}
    }

    set frame [frame $path -class MessageDlg]
    set title [option get $frame "${icon}Title" MessageDlg]
    destroy $frame
    if { $title == "" } {
	set title "Message"
    }

    eval Dialog::create $path $opts -image $image -modal none -title $title \
            -side bottom -anchor c -default 0 -cancel 0
    Dialog::add $path -text [::msgcat::mc "OK"] -name ok -command "destroy $path"

    set frame [Dialog::getframe $path]
    eval message $frame.msg $mopts -relief flat -borderwidth 0 -highlightthickness 0 \
	    -textvariable {{}}
    pack  $frame.msg -side left -padx 3m -pady 1m -fill x -expand yes
    
    Dialog::draw $path
}

proc bindscroll {w} {
    
    bind $w <4> {
    if {[lindex [%W yview] 0] > 0} {
	    %W yview scroll -1 units
	}
    }   
    bind $w <5> {
	if {[lindex [%W yview] 1] < 1} {
	    %W yview scroll 1 units
	}
    }   
    bind $w <6> {
	if {[lindex [%W xview] 0] > 0} {
	    %W xview scroll -1 units
	}
    }
    bind $w <7> {
	if {[lindex [%W xview] 1] < 1} {
	    %W xview scroll 1 units
	}
    }
}
